@@ -1,7 +1,9 @@
+//! @author prozac@rottenboy.com
+
 #include "main.h"
 #include "Client.h"
-#include "User.h"
 #include "znc.h"
+#include "User.h"
 #include "IRCSock.h"
 #include "DCCBounce.h"
 #include "DCCSock.h"
@@ -18,9 +20,7 @@ void CClient::ReadLine(const CString& sData) {
 
 #ifdef _MODULES
 	if (m_bAuthed) {
-		CZNC::Get().GetModules().SetClient(this);
-		MODULECALLRET(OnUserRaw(sLine));
-		CZNC::Get().GetModules().SetClient(NULL);
+		MODULECALL(OnUserRaw(sLine), m_pUser, this, return);
 	}
 #endif
 
@@ -115,6 +115,13 @@ void CClient::ReadLine(const CString& sData) {
 			PutStatusNotice("Detached from [" + sChan + "]");
 			return;
 		}
+	} else if (sCommand.CaseCmp("PING") == 0) {
+		CString sTarget = sLine.Token(1);
+		
+		if (sTarget.CaseCmp("irc.znc.com") == 0) {
+		    PutClient("PONG " + sLine.substr(5));
+		    return;
+		}
 	} else if (sCommand.CaseCmp("PONG") == 0) {
 		return;	// Block pong replies, we already responded to the pings
 	} else if (sCommand.CaseCmp("JOIN") == 0) {
@@ -133,9 +140,7 @@ void CClient::ReadLine(const CString& sData) {
 			for (unsigned int a = 0; a < vChans.size(); a++) {
 				CString sChannel = vChans[a];
 #ifdef _MODULES
-				CZNC::Get().GetModules().SetClient(this);
-				MODULECALLCONT(OnUserJoin(sChannel, sKey));
-				CZNC::Get().GetModules().SetClient(NULL);
+				MODULECALL(OnUserJoin(sChannel, sKey), m_pUser, this, continue);
 #endif
 
 				CChan* pChan = m_pUser->FindChan(sChannel);
@@ -169,9 +174,7 @@ void CClient::ReadLine(const CString& sData) {
 		}
 
 #ifdef _MODULES
-		CZNC::Get().GetModules().SetClient(this);
-		MODULECALLRET(OnUserPart(sChan, sMessage));
-		CZNC::Get().GetModules().SetClient(NULL);
+		MODULECALL(OnUserPart(sChan, sMessage), m_pUser, this, return);
 #endif
 
 		if (m_pUser) {
@@ -189,6 +192,21 @@ void CClient::ReadLine(const CString& sData) {
 		if (!sMessage.empty()) {
 			sLine += " :" + sMessage;
 		}
+	} else if (sCommand.CaseCmp("MODE") == 0) {
+		CString sTarget = sLine.Token(1);
+		CString sModes = sLine.Token(2, true);
+
+		if (m_pUser && m_pUser->IsChan(sTarget)) {
+		    CChan *pChan = m_pUser->FindChan(sTarget);
+
+		    if (pChan && sModes.empty()) {
+			PutClient(":" + m_pUser->GetIRCServer() + " 324 " + GetNick() + " " + sTarget + " " + pChan->GetModeString());
+			if (pChan->GetCreationDate() > 0) {
+			    PutClient(":" + m_pUser->GetIRCServer() + " 329 " + GetNick() + " " + sTarget + " " + CString(pChan->GetCreationDate()));
+			}
+			return;
+		    }
+		}
 	} else if (sCommand.CaseCmp("QUIT") == 0) {
 		if (m_pUser) {
 			m_pUser->UserDisconnected(this);
@@ -196,6 +214,17 @@ void CClient::ReadLine(const CString& sData) {
 
 		Close();	// Treat a client quit as a detach
 		return;		// Don't forward this msg.  We don't want the client getting us disconnected.
+	} else if (sCommand.CaseCmp("PROTOCTL") == 0) {
+		unsigned int i = 1;
+		while(!sLine.Token(i).empty()) {
+			if(sLine.Token(i).CaseCmp("NAMESX") == 0) {
+				m_bNamesx = true;
+			} else if(sLine.Token(i).CaseCmp("UHNAMES") == 0) {
+				m_bUHNames = true;
+			}
+			i++;
+		}
+		return;	// If the server understands it, we already enabled namesx / uhnames
 	} else if (sCommand.CaseCmp("NOTICE") == 0) {
 		CString sTarget = sLine.Token(1);
 		CString sMsg = sLine.Token(2, true);
@@ -242,15 +271,11 @@ void CClient::ReadLine(const CString& sData) {
 			sCTCP.LeftChomp();
 			sCTCP.RightChomp();
 
-			CZNC::Get().GetModules().SetClient(this);
-			MODULECALLRET(OnUserCTCPReply(sTarget, sCTCP));
-			CZNC::Get().GetModules().SetClient(NULL);
+			MODULECALL(OnUserCTCPReply(sTarget, sCTCP), m_pUser, this, return);
 
 			sMsg = "\001" + sCTCP + "\001";
 		} else {
-			CZNC::Get().GetModules().SetClient(this);
-			MODULECALLRET(OnUserNotice(sTarget, sMsg));
-			CZNC::Get().GetModules().SetClient(NULL);
+			MODULECALL(OnUserNotice(sTarget, sMsg), m_pUser, this, return);
 		}
 #endif
 
@@ -260,6 +285,19 @@ void CClient::ReadLine(const CString& sData) {
 			pChan->AddBuffer(":" + GetNickMask() + " NOTICE " + sTarget + " :" + sMsg);
 		}
 
+		// Relay to the rest of the clients that may be connected to this user
+		if (m_pUser && m_pUser->IsChan(sTarget)) {
+			vector<CClient*>& vClients = m_pUser->GetClients();
+
+			for (unsigned int a = 0; a < vClients.size(); a++) {
+				CClient* pClient = vClients[a];
+
+				if (pClient != this) {
+					pClient->PutClient(":" + GetNickMask() + " NOTICE " + sTarget + " :" + sMsg);
+				}
+			}
+		}
+
 		PutIRC("NOTICE " + sTarget + " :" + sMsg);
 		return;
 	} else if (sCommand.CaseCmp("PRIVMSG") == 0) {
@@ -292,7 +330,7 @@ void CClient::ReadLine(const CString& sData) {
 					} else {
 						unsigned short uBNCPort = CDCCBounce::DCCRequest(sTarget, uLongIP, uPort, "", true, m_pUser, (m_pIRCSock) ? m_pIRCSock->GetLocalIP() : GetLocalIP(), "");
 						if (uBNCPort) {
-							PutIRC("PRIVMSG " + sTarget + " :\001DCC CHAT chat " + CString::ToString(CUtils::GetLongIP(sIP)) + " " + CString::ToString(uBNCPort) + "\001");
+							PutIRC("PRIVMSG " + sTarget + " :\001DCC CHAT chat " + CString(CUtils::GetLongIP(sIP)) + " " + CString(uBNCPort) + "\001");
 						}
 					}
 				} else if (sType.CaseCmp("SEND") == 0) {
@@ -320,15 +358,13 @@ void CClient::ReadLine(const CString& sData) {
 							}
 #ifdef _MODULES
 						} else {
-							CZNC::Get().GetModules().SetClient(this);
-							MODULECALLRET(OnDCCUserSend(sTarget, uLongIP, uPort, sFile, uFileSize));
-							CZNC::Get().GetModules().SetClient(NULL);
+							MODULECALL(OnDCCUserSend(sTarget, uLongIP, uPort, sFile, uFileSize), m_pUser, this, return);
 #endif
 						}
 					} else {
 						unsigned short uBNCPort = CDCCBounce::DCCRequest(sTarget, uLongIP, uPort, sFile, false, m_pUser, (m_pIRCSock) ? m_pIRCSock->GetLocalIP() : GetLocalIP(), "");
 						if (uBNCPort) {
-							PutIRC("PRIVMSG " + sTarget + " :\001DCC SEND " + sFile + " " + CString::ToString(CUtils::GetLongIP(sIP)) + " " + CString::ToString(uBNCPort) + " " + CString::ToString(uFileSize) + "\001");
+							PutIRC("PRIVMSG " + sTarget + " :\001DCC SEND " + sFile + " " + CString(CUtils::GetLongIP(sIP)) + " " + CString(uBNCPort) + " " + CString(uFileSize) + "\001");
 						}
 					}
 				} else if (sType.CaseCmp("RESUME") == 0) {
@@ -339,14 +375,14 @@ void CClient::ReadLine(const CString& sData) {
 					// Need to lookup the connection by port, filter the port, and forward to the user
 					if (strncasecmp(sTarget.c_str(), m_pUser->GetStatusPrefix().c_str(), m_pUser->GetStatusPrefix().length()) == 0) {
 						if ((m_pUser) && (m_pUser->ResumeFile(sTarget, uResumePort, uResumeSize))) {
-							PutClient(":" + sTarget + "!znc@znc.com PRIVMSG " + GetNick() + " :\001DCC ACCEPT " + sFile + " " + CString::ToString(uResumePort) + " " + CString::ToString(uResumeSize) + "\001");
+							PutClient(":" + sTarget + "!znc@znc.com PRIVMSG " + GetNick() + " :\001DCC ACCEPT " + sFile + " " + CString(uResumePort) + " " + CString(uResumeSize) + "\001");
 						} else {
 							PutStatus("DCC -> [" + GetNick() + "][" + sFile + "] Unable to find send to initiate resume.");
 						}
 					} else {
 						CDCCBounce* pSock = (CDCCBounce*) CZNC::Get().GetManager().FindSockByLocalPort(uResumePort);
 						if ((pSock) && (strncasecmp(pSock->GetSockName().c_str(), "DCC::", 5) == 0)) {
-							PutIRC("PRIVMSG " + sTarget + " :\001DCC " + sType + " " + sFile + " " + CString::ToString(pSock->GetUserPort()) + " " + sCTCP.Token(4) + "\001");
+							PutIRC("PRIVMSG " + sTarget + " :\001DCC " + sType + " " + sFile + " " + CString(pSock->GetUserPort()) + " " + sCTCP.Token(4) + "\001");
 						}
 					}
 				} else if (sType.CaseCmp("ACCEPT") == 0) {
@@ -360,7 +396,7 @@ void CClient::ReadLine(const CString& sData) {
 
 							if ((pSock) && (strncasecmp(pSock->GetSockName().c_str(), "DCC::", 5) == 0)) {
 								if (pSock->GetUserPort() == atoi(sCTCP.Token(3).c_str())) {
-									PutIRC("PRIVMSG " + sTarget + " :\001DCC " + sType + " " + sFile + " " + CString::ToString(pSock->GetLocalPort()) + " " + sCTCP.Token(4) + "\001");
+									PutIRC("PRIVMSG " + sTarget + " :\001DCC " + sType + " " + sFile + " " + CString(pSock->GetLocalPort()) + " " + sCTCP.Token(4) + "\001");
 								}
 							}
 						}
@@ -393,11 +429,22 @@ void CClient::ReadLine(const CString& sData) {
 				if (pChan && pChan->KeepBuffer()) {
 					pChan->AddBuffer(":" + GetNickMask() + " PRIVMSG " + sTarget + " :\001" + sCTCP + "\001");
 				}
+
+				// Relay to the rest of the clients that may be connected to this user
+				if (m_pUser && m_pUser->IsChan(sTarget)) {
+					vector<CClient*>& vClients = m_pUser->GetClients();
+
+					for (unsigned int a = 0; a < vClients.size(); a++) {
+						CClient* pClient = vClients[a];
+
+						if (pClient != this) {
+							pClient->PutClient(":" + GetNickMask() + " PRIVMSG " + sTarget + " :\001" + sCTCP + "\001");
+						}
+					}
+				}
 #ifdef _MODULES
 			} else {
-				CZNC::Get().GetModules().SetClient(this);
-				MODULECALLRET(OnUserCTCP(sTarget, sCTCP));
-				CZNC::Get().GetModules().SetClient(NULL);
+				MODULECALL(OnUserCTCP(sTarget, sCTCP), m_pUser, this, return);
 #endif
 			}
 
@@ -407,9 +454,7 @@ void CClient::ReadLine(const CString& sData) {
 
 		if ((m_pUser) && (sTarget.CaseCmp(CString(m_pUser->GetStatusPrefix() + "status")) == 0)) {
 #ifdef _MODULES
-			CZNC::Get().GetModules().SetClient(this);
-			MODULECALLRET(OnStatusCommand(sMsg));
-			CZNC::Get().GetModules().SetClient(NULL);
+			MODULECALL(OnStatusCommand(sMsg), m_pUser, this, return);
 #endif
 			UserCommand(sMsg);
 			return;
@@ -445,9 +490,7 @@ void CClient::ReadLine(const CString& sData) {
 		}
 
 #ifdef _MODULES
-		CZNC::Get().GetModules().SetClient(this);
-		MODULECALLRET(OnUserMsg(sTarget, sMsg));
-		CZNC::Get().GetModules().SetClient(NULL);
+		MODULECALL(OnUserMsg(sTarget, sMsg), m_pUser, this, return);
 #endif
 
 		CChan* pChan = m_pUser->FindChan(sTarget);
@@ -531,7 +574,8 @@ void CClient::UserCommand(const CString& sLine) {
 		}
 
 		const map<CString,CNick*>& msNicks = pChan->GetNicks();
-		const CString& sPerms = m_pUser->GetIRCSock()->GetPerms();
+		CIRCSock* pIRCSock = (!m_pUser) ? NULL : m_pUser->GetIRCSock();
+		const CString& sPerms = (pIRCSock) ? pIRCSock->GetPerms() : "";
 
 		if (!msNicks.size()) {
 			PutStatus("No nicks on [" + sChan + "]");
@@ -597,7 +641,11 @@ void CClient::UserCommand(const CString& sLine) {
 			PutStatus("There is no MOTD set.");
 		}
 	} else if (m_pUser->IsAdmin() && sCommand.CaseCmp("SaveConfig") == 0) {
-		CZNC::Get().WriteConfig();
+		if (CZNC::Get().WriteConfig()) {
+			PutStatus("Wrote config to [" + CZNC::Get().GetConfigFile() + "]");
+		} else {
+			PutStatus("Error while trying to write config.");
+		}
 	} else if (sCommand.CaseCmp("LISTCLIENTS") == 0) {
 		if (m_pUser) {
 			CUser* pUser = m_pUser;
@@ -653,7 +701,7 @@ void CClient::UserCommand(const CString& sLine) {
 		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
 			Table.AddRow();
 			Table.SetCell("Username", it->first);
-			Table.SetCell("Clients", CString::ToString(it->second->GetClients().size()));
+			Table.SetCell("Clients", CString(it->second->GetClients().size()));
 			if (!it->second->IsIRCConnected()) {
 				Table.SetCell("OnIRC", "No");
 			} else {
@@ -716,7 +764,8 @@ void CClient::UserCommand(const CString& sLine) {
 	} else if (sCommand.CaseCmp("LISTCHANS") == 0) {
 		if (m_pUser) {
 			const vector<CChan*>& vChans = m_pUser->GetChans();
-			const CString& sPerms = m_pUser->GetIRCSock()->GetPerms();
+			CIRCSock* pIRCSock = (!m_pUser) ? NULL : m_pUser->GetIRCSock();
+			const CString& sPerms = (pIRCSock) ? pIRCSock->GetPerms() : "";
 
 			if (!vChans.size()) {
 				PutStatus("You have no channels defined");
@@ -743,14 +792,14 @@ void CClient::UserCommand(const CString& sLine) {
 				Table.SetCell("Name", pChan->GetPermStr() + pChan->GetName());
 				Table.SetCell("Status", ((vChans[a]->IsOn()) ? ((vChans[a]->IsDetached()) ? "Detached" : "Joined") : "Trying"));
 				Table.SetCell("Conf", CString((pChan->InConfig()) ? "yes" : ""));
-				Table.SetCell("Buf", CString((pChan->KeepBuffer()) ? "*" : "") + CString::ToString(pChan->GetBufferCount()));
+				Table.SetCell("Buf", CString((pChan->KeepBuffer()) ? "*" : "") + CString(pChan->GetBufferCount()));
 				Table.SetCell("Modes", pChan->GetModeString());
-				Table.SetCell("Users", CString::ToString(pChan->GetNickCount()));
+				Table.SetCell("Users", CString(pChan->GetNickCount()));
 
 				for (unsigned int b = 0; b < sPerms.size(); b++) {
 					CString sPerm;
 					sPerm += sPerms[b];
-					Table.SetCell(sPerm, CString::ToString(pChan->GetPermCount(sPerms[b])));
+					Table.SetCell(sPerm, CString(pChan->GetPermCount(sPerms[b])));
 				}
 			}
 
@@ -791,8 +840,8 @@ void CClient::UserCommand(const CString& sLine) {
 
 		const vector<CServer*>& vServers = m_pUser->GetServers();
 
-		if (vServers.size() <= 1) {
-			PutStatus("You must have at least one server at all times.");
+		if (vServers.size() <= 0) {
+			PutStatus("You don't have any servers added.");
 			return;
 		}
 
@@ -803,6 +852,7 @@ void CClient::UserCommand(const CString& sLine) {
 		}
 	} else if (sCommand.CaseCmp("LISTSERVERS") == 0) {
 		if (m_pUser) {
+		    if (m_pUser->HasServers()) {
 			const vector<CServer*>& vServers = m_pUser->GetServers();
 			CTable Table;
 			Table.AddColumn("Host");
@@ -814,7 +864,7 @@ void CClient::UserCommand(const CString& sLine) {
 				CServer* pServer = vServers[a];
 				Table.AddRow();
 				Table.SetCell("Host", pServer->GetName());
-				Table.SetCell("Port", CString::ToString(pServer->GetPort()));
+				Table.SetCell("Port", CString(pServer->GetPort()));
 				Table.SetCell("SSL", (pServer->IsSSL()) ? "SSL" : "");
 				Table.SetCell("Pass", pServer->GetPass());
 			}
@@ -827,6 +877,9 @@ void CClient::UserCommand(const CString& sLine) {
 					PutStatus(sLine);
 				}
 			}
+		    } else {
+			PutStatus("You don't have any servers added.");
+		    }
 		}
 	} else if (sCommand.CaseCmp("TOPICS") == 0) {
 		if (m_pUser) {
@@ -856,24 +909,37 @@ void CClient::UserCommand(const CString& sLine) {
 	} else if (sCommand.CaseCmp("SEND") == 0) {
 		CString sToNick = sLine.Token(1);
 		CString sFile = sLine.Token(2);
+		CString sAllowedPath = m_pUser->GetDLPath();
+		CString sAbsolutePath;
 
 		if ((sToNick.empty()) || (sFile.empty())) {
 			PutStatus("Usage: Send <nick> <file>");
 			return;
 		}
 
+		sAbsolutePath = CUtils::ChangeDir(m_pUser->GetDLPath(), sFile, CZNC::Get().GetHomePath());
+
+		if (sAbsolutePath.Left(sAllowedPath.length()) != sAllowedPath) {
+			PutStatus("Illegal path.");
+			return;
+		}
+
 		if (m_pUser) {
 			m_pUser->SendFile(sToNick, sFile);
 		}
 	} else if (sCommand.CaseCmp("GET") == 0) {
 		CString sFile = sLine.Token(1);
+		CString sAllowedPath = m_pUser->GetDLPath();
+		CString sAbsolutePath;
 
 		if (sFile.empty()) {
 			PutStatus("Usage: Get <file>");
 			return;
 		}
 
-		if (sFile.find("..") != CString::npos) {
+		sAbsolutePath = CUtils::ChangeDir(m_pUser->GetDLPath(), sFile, CZNC::Get().GetHomePath());
+
+		if (sAbsolutePath.Left(sAllowedPath.length()) != sAllowedPath) {
 			PutStatus("Illegal path.");
 			return;
 		}
@@ -982,6 +1048,30 @@ void CClient::UserCommand(const CString& sLine) {
 		}
 	} else if ((sCommand.CaseCmp("LISTMODS") == 0) || (sCommand.CaseCmp("LISTMODULES") == 0)) {
 #ifdef _MODULES
+		if (m_pUser->IsAdmin()) {
+		    CModules& GModules = CZNC::Get().GetModules();
+
+		    if (!GModules.size()) {
+				PutStatus("No global modules loaded.");
+			} else {
+				CTable GTable;
+				GTable.AddColumn("Name");
+				GTable.AddColumn("Description");
+
+				for (unsigned int b = 0; b < GModules.size(); b++) {
+					GTable.AddRow();
+					GTable.SetCell("Name", GModules[b]->GetModName());
+					GTable.SetCell("Description", GModules[b]->GetDescription().Ellipsize(128));
+				}
+
+				unsigned int uTableIdx = 0; CString sLine;
+
+				while (GTable.GetLine(uTableIdx++, sLine)) {
+					PutStatus(sLine);
+				}
+			}
+		}
+
 		if (m_pUser) {
 			CModules& Modules = m_pUser->GetModules();
 
@@ -1010,8 +1100,24 @@ void CClient::UserCommand(const CString& sLine) {
 #endif
 		return;
 	} else if ((sCommand.CaseCmp("LOADMOD") == 0) || (sCommand.CaseCmp("LOADMODULE") == 0)) {
-		CString sMod = sLine.Token(1);
-		CString sArgs = sLine.Token(2, true);
+		CString sMod;
+		CString sArgs;
+		bool bGlobal = false;
+
+		if (sLine.Token(1).CaseCmp("-global") == 0) {
+		    sMod = sLine.Token(2);
+
+		    if (!m_pUser->IsAdmin()) {
+				PutStatus("Unable to load global module [" + sMod + "] Access Denied.");
+				return;
+		    }
+
+		    sArgs = sLine.Token(3, true);
+		    bGlobal = true;
+		} else {
+		    sMod = sLine.Token(1);
+		    sArgs = sLine.Token(2, true);
+		}
 
 		if (m_pUser->DenyLoadMod()) {
 			PutStatus("Unable to load [" + sMod + "] Access Denied.");
@@ -1019,14 +1125,18 @@ void CClient::UserCommand(const CString& sLine) {
 		}
 #ifdef _MODULES
 		if (sMod.empty()) {
-			PutStatus("Usage: LoadMod <module> [args]");
+			PutStatus("Usage: LoadMod [-global] <module> [args]");
 			return;
 		}
 
 		CString sModRet;
 
 		try {
-			m_pUser->GetModules().LoadModule(sMod, sArgs, m_pUser, sModRet);
+			if (bGlobal) {
+			    CZNC::Get().GetModules().LoadModule(sMod, sArgs, NULL, sModRet);
+			} else {
+			    m_pUser->GetModules().LoadModule(sMod, sArgs, m_pUser, sModRet);
+			}
 		} catch (CException e) {
 			PutStatus("Unable to load module [" + sMod + "] [" + sModRet + "]");
 			return;
@@ -1038,7 +1148,20 @@ void CClient::UserCommand(const CString& sLine) {
 #endif
 		return;
 	} else if ((sCommand.CaseCmp("UNLOADMOD") == 0) || (sCommand.CaseCmp("UNLOADMODULE") == 0)) {
-		CString sMod = sLine.Token(1);
+		CString sMod;
+		bool bGlobal = false;
+
+		if (sLine.Token(1).CaseCmp("-global") == 0) {
+		    sMod = sLine.Token(2);
+
+		    if (!m_pUser->IsAdmin()) {
+				PutStatus("Unable to unload global module [" + sMod + "] Access Denied.");
+				return;
+		    }
+
+		    bGlobal = true;
+		} else
+		    sMod = sLine.Token(1);
 
 		if (m_pUser->DenyLoadMod()) {
 			PutStatus("Unable to unload [" + sMod + "] Access Denied.");
@@ -1046,20 +1169,42 @@ void CClient::UserCommand(const CString& sLine) {
 		}
 #ifdef _MODULES
 		if (sMod.empty()) {
-			PutStatus("Usage: UnloadMod <module>");
+			PutStatus("Usage: UnloadMod [-global] <module>");
 			return;
 		}
 
 		CString sModRet;
-		m_pUser->GetModules().UnloadModule(sMod, sModRet);
+
+		if (bGlobal) {
+		    CZNC::Get().GetModules().UnloadModule(sMod, sModRet);
+		} else {
+		    m_pUser->GetModules().UnloadModule(sMod, sModRet);
+		}
+
 		PutStatus(sModRet);
 #else
 		PutStatus("Unable to unload [" + sMod + "] Modules are not enabled.");
 #endif
 		return;
 	} else if ((sCommand.CaseCmp("RELOADMOD") == 0) || (sCommand.CaseCmp("RELOADMODULE") == 0)) {
-		CString sMod = sLine.Token(1);
-		CString sArgs = sLine.Token(2, true);
+		CString sMod;
+		CString sArgs;
+		bool bGlobal = false;
+
+		if (sLine.Token(1).CaseCmp("-global") == 0) {
+		    sMod = sLine.Token(2);
+
+		    if (!m_pUser->IsAdmin()) {
+				PutStatus("Unable to reload global module [" + sMod + "] Access Denied.");
+				return;
+		    }
+
+		    sArgs = sLine.Token(3, true);
+		    bGlobal = true;
+		} else {
+		    sMod = sLine.Token(1);
+		    sArgs = sLine.Token(2, true);
+		}
 
 		if (m_pUser->DenyLoadMod()) {
 			PutStatus("Unable to reload [" + sMod + "] Access Denied.");
@@ -1067,17 +1212,80 @@ void CClient::UserCommand(const CString& sLine) {
 		}
 #ifdef _MODULES
 		if (sMod.empty()) {
-			PutStatus("Usage: ReloadMod <module> [args]");
+			PutStatus("Usage: ReloadMod [-global] <module> [args]");
 			return;
 		}
 
 		CString sModRet;
-		m_pUser->GetModules().ReloadModule(sMod, sArgs, m_pUser, sModRet);
+
+		if (bGlobal) {
+		    CZNC::Get().GetModules().ReloadModule(sMod, sArgs, NULL, sModRet);
+		} else {
+		    m_pUser->GetModules().ReloadModule(sMod, sArgs, m_pUser, sModRet);
+		}
+
 		PutStatus(sModRet);
 #else
 		PutStatus("Unable to unload [" + sMod + "] Modules are not enabled.");
 #endif
 		return;
+	} else if (sCommand.CaseCmp("SETVHOST") == 0) {
+		CString sVHost = sLine.Token(1);
+
+		if (sVHost.empty()) {
+			PutStatus("Usage: SetVHost <VHost>");
+		}
+
+		m_pUser->SetVHost(sVHost);
+		PutStatus("Set VHost to [" + m_pUser->GetVHost() + "]");
+	} else if (sCommand.CaseCmp("PLAYBUFFER") == 0) {
+		CString sChan = sLine.Token(1);
+
+		if (sChan.empty()) {
+			PutStatus("Usage: PlayBuffer <#chan>");
+			return;
+		}
+
+		CChan* pChan = m_pUser->FindChan(sChan);
+
+		if (!pChan) {
+			PutStatus("You are not on [" + sChan + "]");
+			return;
+		}
+
+		if (!pChan->IsOn()) {
+			PutStatus("You are not on [" + sChan + "] [trying]");
+			return;
+		}
+
+		if (pChan->GetBuffer().empty()) {
+			PutStatus("The buffer for [" + sChan + "] is empty");
+			return;
+		}
+
+		pChan->SendBuffer(this);
+	} else if (sCommand.CaseCmp("CLEARBUFFER") == 0) {
+		CString sChan = sLine.Token(1);
+
+		if (sChan.empty()) {
+			PutStatus("Usage: ClearBuffer <#chan>");
+			return;
+		}
+
+		CChan* pChan = m_pUser->FindChan(sChan);
+
+		if (!pChan) {
+			PutStatus("You are not on [" + sChan + "]");
+			return;
+		}
+
+		if (!pChan->IsOn()) {
+			PutStatus("You are not on [" + sChan + "] [trying]");
+			return;
+		}
+
+		pChan->ClearBuffer();
+		PutStatus("The buffer for [" + sChan + "] has been cleared");
 	} else if (sCommand.CaseCmp("SETBUFFER") == 0) {
 		CString sChan = sLine.Token(1);
 
@@ -1107,7 +1315,7 @@ void CClient::UserCommand(const CString& sLine) {
 
 		pChan->SetBufferCount(uLineCount);
 
-		PutStatus("BufferCount for [" + sChan + "] set to [" + CString::ToString(pChan->GetBufferCount()) + "]");
+		PutStatus("BufferCount for [" + sChan + "] set to [" + CString(pChan->GetBufferCount()) + "]");
 	} else {
 		PutStatus("Unknown command [" + sCommand + "] try 'Help'");
 	}
@@ -1140,14 +1348,17 @@ void CClient::HelpUser() {
 	Table.AddRow(); Table.SetCell("Command", "ListNicks");	Table.SetCell("Arguments", "<#chan>");				Table.SetCell("Description", "List all nicks on a channel");
 
 	if (!m_pUser->IsAdmin()) { // If they are an admin we will add this command below with an argument
-		Table.AddRow(); Table.SetCell("Command", "ListClients");Table.SetCell("Arguments", "");						Table.SetCell("Description", "List all clients connected to your znc user");
+		Table.AddRow(); Table.SetCell("Command", "ListClients");Table.SetCell("Arguments", "");					Table.SetCell("Description", "List all clients connected to your znc user");
 	}
 
 	Table.AddRow(); Table.SetCell("Command", "ListServers");Table.SetCell("Arguments", "");						Table.SetCell("Description", "List all servers");
 	Table.AddRow(); Table.SetCell("Command", "AddServer");	Table.SetCell("Arguments", "<host> [[+]port] [pass]");	Table.SetCell("Description", "Add a server to the list");
 	Table.AddRow(); Table.SetCell("Command", "RemServer");	Table.SetCell("Arguments", "<host>");				Table.SetCell("Description", "Remove a server from the list");
 	Table.AddRow(); Table.SetCell("Command", "Topics");		Table.SetCell("Arguments", "");						Table.SetCell("Description", "Show topics in all channels");
+	Table.AddRow(); Table.SetCell("Command", "PlayBuffer");	Table.SetCell("Arguments", "<#chan>");				Table.SetCell("Description", "Play back the buffer for a given channel");
+	Table.AddRow(); Table.SetCell("Command", "ClearBuffer");Table.SetCell("Arguments", "<#chan>");				Table.SetCell("Description", "Clear the buffer for a given channel");
 	Table.AddRow(); Table.SetCell("Command", "SetBuffer");	Table.SetCell("Arguments", "<#chan> [linecount]");	Table.SetCell("Description", "Set the buffer count for a channel");
+	Table.AddRow(); Table.SetCell("Command", "SetVHost");	Table.SetCell("Arguments", "<vhost (ip preferred)>");Table.SetCell("Description", "Set the VHost for this connection");
 	Table.AddRow(); Table.SetCell("Command", "Jump");		Table.SetCell("Arguments", "");						Table.SetCell("Description", "Jump to the next server in the list");
 	Table.AddRow(); Table.SetCell("Command", "Send");		Table.SetCell("Arguments", "<nick> <file>");		Table.SetCell("Description", "Send a shell file to a nick on IRC");
 	Table.AddRow(); Table.SetCell("Command", "Get");		Table.SetCell("Arguments", "<file>");				Table.SetCell("Description", "Send a shell file to yourself");
@@ -1187,39 +1398,92 @@ bool CClient::ConnectionFrom(const CString& sHost, unsigned short uPort) {
 }
 
 void CClient::AuthUser() {
+	/*
+#ifdef _MODULES
+	if (CZNC::Get().GetModules().OnLoginAttempt(m_sUser, m_sPass, *this)) {
+		return;
+	}
+#endif
+
 	CUser* pUser = CZNC::Get().GetUser(m_sUser);
 
-	if ((!pUser) || (!pUser->CheckPass(m_sPass))) {
-		if (pUser) {
-			pUser->PutStatus("Another user attempted to login as you, with a bad password.");
-		}
-
-		PutStatus("Bad username and/or password.");
-		PutClient(":irc.znc.com 464 " + GetNick() + " :Password Incorrect");
-		Close();
+	if (pUser && pUser->CheckPass(m_sPass)) {
+		AcceptLogin(*pUser);
 	} else {
-		m_sPass = "";
-		m_pUser = pUser;
-
-		if (!m_pUser->IsHostAllowed(GetRemoteIP())) {
-			Close();
-			return;
+		if (pUser) {
+			pUser->PutStatus("Another client attempted to login as you, with a bad password.");
 		}
 
-		m_bAuthed = true;
-		SetSockName("USR::" + pUser->GetUserName());
+		RefuseLogin();
+	}
+	*/
 
-		m_pIRCSock = (CIRCSock*) CZNC::Get().FindSockByName("IRC::" + pUser->GetUserName());
-		m_pUser->UserConnected(this);
+	m_spAuth = new CClientAuth(this, m_sUser, m_sPass);
 
-		SendMotd();
+	CClientAuth::AuthUser(m_spAuth);
+}
+
+void CAuthBase::AuthUser(CSmartPtr<CAuthBase> AuthClass) {
+#ifdef _MODULES
+	if (CZNC::Get().GetModules().OnLoginAttempt(AuthClass)) {
+		return;
+	}
+#endif
+
+	CUser* pUser = CZNC::Get().GetUser(AuthClass->GetUsername());
+
+	if (pUser && pUser->CheckPass(AuthClass->GetPassword())) {
+		AuthClass->AcceptLogin(*pUser);
+	} else {
+		if (pUser) {
+			pUser->PutStatus("Another client attempted to login as you, with a bad password.");
+		}
+
+		AuthClass->RefuseLogin("Invalid Password");
+	}
+}
+
+void CClientAuth::RefuseLogin(const CString& sReason) {
+	if (m_pClient) {
+		m_pClient->RefuseLogin(sReason);
+	}
+#ifdef _MODULES
+	CZNC::Get().GetModules().OnFailedLogin(GetUsername(), ((m_pClient) ? m_pClient->GetRemoteIP() : CString("")));
+#endif
+}
+
+void CClient::RefuseLogin(const CString& sReason) {
+	PutStatus("Bad username and/or password.");
+	PutClient(":irc.znc.com 464 " + GetNick() + " :" + sReason);
+	Close();
+}
+
+void CClientAuth::AcceptLogin(CUser& User) {
+	if (m_pClient) {
+		m_pClient->AcceptLogin(User);
+	}
+}
+
+void CClient::AcceptLogin(CUser& User) {
+	m_sPass = "";
+	m_pUser = &User;
+
+	if (!m_pUser->IsHostAllowed(GetRemoteIP())) {
+		Close();
+		return;
+	}
+
+	m_bAuthed = true;
+	SetSockName("USR::" + m_pUser->GetUserName());
+
+	m_pIRCSock = (CIRCSock*) CZNC::Get().FindSockByName("IRC::" + m_pUser->GetUserName());
+	m_pUser->UserConnected(this);
+
+	SendMotd();
 
 #ifdef _MODULES
-		CZNC::Get().GetModules().SetClient(this);
-		VOIDMODULECALL(OnUserAttached());
-		CZNC::Get().GetModules().SetClient(NULL);
+	MODULECALL(OnUserAttached(), m_pUser, this, );
 #endif
-	}
 }
 
 void CClient::Connected() {
@@ -1239,9 +1503,7 @@ void CClient::Disconnected() {
 	m_pIRCSock = NULL;
 
 #ifdef _MODULES
-	CZNC::Get().GetModules().SetClient(this);
-	VOIDMODULECALL(OnUserDetached());
-	CZNC::Get().GetModules().SetClient(NULL);
+	MODULECALL(OnUserDetached(), m_pUser, this, );
 #endif
 }
 
