@@ -1,6 +1,6 @@
-/**
+/** @file
 *
-*    Copyright (c) 1999-2005 Jim Hull <imaginos@imaginos.net>
+*    Copyright (c) 1999-2007 Jim Hull <imaginos@imaginos.net>
 *    All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without modification,
@@ -11,7 +11,7 @@
 * of conditions and the following disclaimer in the documentation and/or other materials
 *  provided with the distribution.
 * Redistributions in any form must be accompanied by information on how to obtain
-* complete source code for the DB software and any accompanying software that uses the DB software.
+* complete source code for this software and any accompanying software that uses this software.
 * The source code must either be included in the distribution or be available for no more than
 * the cost of distribution plus a nominal fee, and must be freely redistributable
 * under reasonable conditions. For an executable file, complete source code means the source
@@ -20,7 +20,7 @@
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
-* OR NON-INFRINGEMENT, ARE DISCLAIMED. IN NO EVENT SHALL SLEEPYCAT SOFTWARE BE LIABLE FOR ANY DIRECT,
+* OR NON-INFRINGEMENT, ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OF THIS SOFTWARE BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.14 $
+* $Revision: 1.21 $
 */
 
 #include "Csocket.h"
@@ -45,7 +45,8 @@ int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockA
 	struct addrinfo *res = NULL;
 	struct addrinfo hints;
 	memset( (struct addrinfo *)&hints, '\0', sizeof( hints ) );
-	hints.ai_family = PF_UNSPEC;
+	hints.ai_family = csSockAddr.GetAFRequire();
+
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_protocol = IPPROTO_TCP;
 	int iRet = getaddrinfo( sHostname.c_str(), NULL, &hints, &res );
@@ -56,8 +57,16 @@ int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockA
 		bool bFoundEntry = false;
 		for( struct addrinfo *pRes = res; pRes; pRes = pRes->ai_next )
 		{
+#ifdef __sun
+			if( ( pRes->ai_socktype != SOCK_STREAM ) || ( pRes->ai_protocol != IPPROTO_TCP && pRes->ai_protocol != IPPROTO_IP ) )
+#else
 			if( ( pRes->ai_socktype != SOCK_STREAM ) || ( pRes->ai_protocol != IPPROTO_TCP ) )
+#endif /* __sun work around broken impl of getaddrinfo */
 				continue;
+
+			if( ( csSockAddr.GetAFRequire() != CSSockAddr::RAF_ANY ) && ( pRes->ai_family != csSockAddr.GetAFRequire() ) )
+				continue; // they requested a special type, so be certain we woop past anything unwanted
+
 			if( pRes->ai_family == AF_INET )
 			{
 				if( pSock )
@@ -294,27 +303,25 @@ CCron::CCron()
 	m_bPause = false;
 }
 
-//! This is used by the Job Manager, and not you directly
-void CCron::run()
+void CCron::run( time_t & iNow )
 {
 	if ( m_bPause )
 		return;
 
-	if ( ( m_bActive ) && ( time( NULL ) >= m_iTime ) )
+	if( iNow == 0 )
+		iNow = time( NULL );
+
+	if ( ( m_bActive ) && ( iNow >= m_iTime ) )
 	{
 		RunJob();
 
 		if ( ( m_iMaxCycles > 0 ) && ( ++m_iCycles >= m_iMaxCycles  ) )
 			m_bActive = false;
 		else
-			m_iTime = time( NULL ) + m_iTimeSequence;
+			m_iTime = iNow + m_iTimeSequence;
 	}
 }
 
-/**
- * @TimeSequence	how often to run in seconds
- * @iMaxCycles		how many times to run, 0 makes it run forever
- */
 void CCron::StartMaxCycles( int TimeSequence, u_int iMaxCycles )
 {
 	m_iTimeSequence = TimeSequence;
@@ -322,7 +329,6 @@ void CCron::StartMaxCycles( int TimeSequence, u_int iMaxCycles )
 	m_iMaxCycles = iMaxCycles;
 }
 
-//! starts and runs infinity amount of times
 void CCron::Start( int TimeSequence )
 {
 	m_iTimeSequence = TimeSequence;
@@ -330,19 +336,16 @@ void CCron::Start( int TimeSequence )
 	m_iMaxCycles = 0;
 }
 
-//! call this to turn off your cron, it will be removed
 void CCron::Stop()
 {
 	m_bActive = false;
 }
 
-//! pauses excution of your code in RunJob
 void CCron::Pause()
 {
 	m_bPause = true;
 }
 
-//! removes the pause on RunJon
 void CCron::UnPause()
 {
 	m_bPause = false;
@@ -352,7 +355,6 @@ int CCron::GetInterval() const { return( m_iTimeSequence ); }
 u_int CCron::GetMaxCycles() const { return( m_iMaxCycles ); }
 u_int CCron::GetCyclesLeft() const { return( ( m_iMaxCycles > m_iCycles ? ( m_iMaxCycles - m_iCycles ) : 0 ) ); }
 
-//! returns true if cron is active
 bool CCron::isValid() { return( m_bActive ); }
 const CS_STRING & CCron::GetName() const { return( m_sName ); }
 void CCron::SetName( const CS_STRING & sName ) { m_sName = sName; }
@@ -360,11 +362,17 @@ void CCron::RunJob() { CS_DEBUG( "This should be overriden" ); }
 
 Csock::Csock( int itimeout )
 {
+#ifdef HAVE_LIBSSL
+	m_pCerVerifyCB = NULL;
+#endif /* HAVE_LIBSSL */
 	Init( "", 0, itimeout );
 }
 
 Csock::Csock( const CS_STRING & sHostname, u_short iport, int itimeout )
 {
+#ifdef HAVE_LIBSSL
+	m_pCerVerifyCB = NULL;
+#endif /* HAVE_LIBSSL */
 	Init( sHostname, iport, itimeout );
 }
 
@@ -755,7 +763,6 @@ bool Csock::AcceptSSL()
 	return( false );
 }
 
-//! This sets up the SSL Client, this is used internally
 bool Csock::SSLClientSetup()
 {
 #ifdef HAVE_LIBSSL
@@ -825,8 +832,9 @@ bool Csock::SSLClientSetup()
 
 	SSL_set_rfd( m_ssl, m_iReadSock );
 	SSL_set_wfd( m_ssl, m_iWriteSock );
-	SSL_set_verify( m_ssl, SSL_VERIFY_PEER, CertVerifyCB );
+	SSL_set_verify( m_ssl, SSL_VERIFY_PEER, ( m_pCerVerifyCB ? m_pCerVerifyCB : CertVerifyCB ) );
 
+	SSLFinishSetup( m_ssl );
 	return( true );
 #else
 	return( false );
@@ -919,10 +927,10 @@ bool Csock::SSLServerSetup()
 	SSL_set_rfd( m_ssl, m_iReadSock );
 	SSL_set_wfd( m_ssl, m_iWriteSock );
 	SSL_set_accept_state( m_ssl );
-
 	if ( m_bRequireClientCert )
-		SSL_set_verify( m_ssl, SSL_VERIFY_FAIL_IF_NO_PEER_CERT|SSL_VERIFY_PEER, CertVerifyCB );
+		SSL_set_verify( m_ssl, SSL_VERIFY_FAIL_IF_NO_PEER_CERT|SSL_VERIFY_PEER, ( m_pCerVerifyCB ? m_pCerVerifyCB : CertVerifyCB ) );
 
+	SSLFinishSetup( m_ssl );
 	return( true );
 #else
 	return( false );
@@ -983,6 +991,21 @@ bool Csock::ConnectSSL( const CS_STRING & sBindhost )
 #endif /* HAVE_LIBSSL */
 }
 
+bool Csock::AllowWrite( unsigned long long & iNOW ) const
+{
+	if ( ( m_iMaxBytes > 0 ) && ( m_iMaxMilliSeconds > 0 ) )
+	{
+		if( iNOW == 0 )
+			iNOW = millitime();
+
+		if( m_iLastSend <  m_iMaxBytes )
+			return( true ); // allow sending if our out buffer was less than what we can send
+		if ( ( iNOW - m_iLastSendTime ) < m_iMaxMilliSeconds )
+			return( false );
+	}
+	return( true );
+}
+
 bool Csock::Write( const char *data, int len )
 {
 	m_sSend.append( data, len );
@@ -1187,7 +1210,8 @@ int Csock::Read( char *data, int len )
 #endif /* HAVE_LIBSSL */
 	}
 
-	m_iBytesRead += (unsigned long long)bytes;
+	if( bytes > 0 ) // becareful not to add negative bytes :P
+		m_iBytesRead += (unsigned long long)bytes;
 
 	return( bytes );
 }
@@ -1280,6 +1304,7 @@ void Csock::UnPauseRead()
 {
 	m_bPauseRead = false;
 	ResetTimer();
+	PushBuff( "", 0, true );
 }
 
 void Csock::SetTimeout( int iTimeout, u_int iTimeoutType )
@@ -1292,7 +1317,6 @@ void Csock::SetTimeoutType( u_int iTimeoutType ) { m_iTimeoutType = iTimeoutType
 int Csock::GetTimeout() const { return m_itimeout; }
 u_int Csock::GetTimeoutType() const { return( m_iTimeoutType ); }
 
-//! returns true if the socket has timed out
 bool Csock::CheckTimeout()
 {
 	if ( IsReadPaused() )
@@ -1311,17 +1335,17 @@ bool Csock::CheckTimeout()
 	return( false );
 }
 
-void Csock::PushBuff( const char *data, int len )
+void Csock::PushBuff( const char *data, int len, bool bStartAtZero )
 {
 	if ( !m_bEnableReadLine )
 		return;	// If the ReadLine event is disabled, just ditch here
 
-	u_int iStartPos = ( m_sbuffer.empty() ? 0 : m_sbuffer.length() - 1 );
+	u_int iStartPos = ( m_sbuffer.empty() || bStartAtZero ? 0 : m_sbuffer.length() - 1 );
 
 	if ( data )
 		m_sbuffer.append( data, len );
 
-	while( true )
+	while( !m_bPauseRead )
 	{
 		CS_STRING::size_type iFind = m_sbuffer.find( "\n", iStartPos );
 
@@ -1469,7 +1493,6 @@ void Csock::NonBlockingIO()
 	BlockIO( false );
 }
 
-//! if this connection type is ssl or not
 bool Csock::GetSSL() { return( m_bssl ); }
 void Csock::SetSSL( bool b ) { m_bssl = b; }
 
@@ -1497,7 +1520,6 @@ int Csock::CertVerifyCB( int preverify_ok, X509_STORE_CTX *x509_ctx )
 	return( 1 );
 }
 
-//! Set the SSL method type
 void Csock::SetSSLMethod( int iMethod ) { m_iMethod = iMethod; }
 int Csock::GetSSLMethod() { return( m_iMethod ); }
 void Csock::SetSSLObject( SSL *ssl ) { m_ssl = ssl; }
@@ -1641,6 +1663,8 @@ unsigned long long Csock::GetRateTime() { return( m_iMaxMilliSeconds ); }
 
 void Csock::Cron()
 {
+	time_t iNow = 0;
+
 	for( vector<CCron *>::size_type a = 0; a < m_vcCrons.size(); a++ )
 	{
 		CCron *pcCron = m_vcCrons[a];
@@ -1650,17 +1674,15 @@ void Csock::Cron()
 			CS_Delete( pcCron );
 			m_vcCrons.erase( m_vcCrons.begin() + a-- );
 		} else
-			pcCron->run();
+			pcCron->run( iNow );
 	}
 }
 
-//! insert a newly created cron
 void Csock::AddCron( CCron * pcCron )
 {
 	m_vcCrons.push_back( pcCron );
 }
 
-//! delete cron(s) by name
 void Csock::DelCron( const CS_STRING & sName, bool bDeleteAll, bool bCaseSensitive )
 {
 	for( u_int a = 0; a < m_vcCrons.size(); a++ )
@@ -1675,7 +1697,6 @@ void Csock::DelCron( const CS_STRING & sName, bool bDeleteAll, bool bCaseSensiti
 	}
 }
 
-//! delete cron by idx
 void Csock::DelCron( u_int iPos )
 {
 	if ( iPos < m_vcCrons.size() )
@@ -1685,7 +1706,7 @@ void Csock::DelCron( u_int iPos )
 		m_vcCrons.erase( m_vcCrons.begin() + iPos );
 	}
 }
-//! delete cron by address
+
 void Csock::DelCronByAddr( CCron *pcCron )
 {
 	for( u_int a = 0; a < m_vcCrons.size(); a++ )
@@ -1881,10 +1902,15 @@ void Csock::FREE_CTX()
 
 #endif /* HAVE_LIBSSL */
 
-//! Create the socket
 int Csock::SOCKET( bool bListen )
 {
+#ifdef HAVE_IPV6
 	int iRet = socket( ( GetIPv6() ? PF_INET6 : PF_INET ), SOCK_STREAM, IPPROTO_TCP );
+#else
+	// missing wrapper around ipv6 for systems missing ipv6, Uli Schlachter <psycho@foex-gaming.com>
+	int iRet = socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
+#endif /* HAVE_IPV6 */
+
 
 	if ( ( iRet > -1 ) && ( bListen ) )
 	{
