@@ -1,6 +1,6 @@
 /**
 *
-*    Copyright (c) 1999-2005 Jim Hull <imaginos@imaginos.net>
+*    Copyright (c) 1999-2007 Jim Hull <imaginos@imaginos.net>
 *    All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without modification,
@@ -11,7 +11,7 @@
 * of conditions and the following disclaimer in the documentation and/or other materials
 *  provided with the distribution.
 * Redistributions in any form must be accompanied by information on how to obtain
-* complete source code for the DB software and any accompanying software that uses the DB software.
+* complete source code for this software and any accompanying software that uses this software.
 * The source code must either be included in the distribution or be available for no more than
 * the cost of distribution plus a nominal fee, and must be freely redistributable
 * under reasonable conditions. For an executable file, complete source code means the source
@@ -20,7 +20,7 @@
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
-* OR NON-INFRINGEMENT, ARE DISCLAIMED. IN NO EVENT SHALL SLEEPYCAT SOFTWARE BE LIABLE FOR ANY DIRECT,
+* OR NON-INFRINGEMENT, ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OF THIS SOFTWARE BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.20 $
+* $Revision: 1.27 $
 */
 
 // note to compile with win32 need to link to winsock2, using gcc its -lws2_32
@@ -122,10 +122,20 @@ public:
 #ifdef HAVE_IPV6
 		memset( (struct sockaddr_in6 *)&m_saddr6, '\0', sizeof( m_saddr6 ) );
 #endif /* HAVE_IPV6 */
+		m_iAFRequire = RAF_ANY;
 	}
 	virtual ~CSSockAddr() {}
 
 
+	enum EAFRequire
+	{
+		RAF_ANY		= PF_UNSPEC,
+#ifdef HAVE_IPV6
+		RAF_INET6	= AF_INET6,
+#endif /* HAVE_IPV6 */
+		RAF_INET	= AF_INET
+	};
+
 	void SinFamily()
 	{
 #ifdef HAVE_IPV6
@@ -165,7 +175,6 @@ public:
 	bool GetIPv6() const { return( m_bIsIPv6 ); }
 
 
-
 	socklen_t GetSockAddrLen() { return( sizeof( m_saddr ) ); }
 	sockaddr_in * GetSockAddr() { return( &m_saddr ); }
 	in_addr * GetAddr() { return( &(m_saddr.sin_addr) ); }
@@ -175,13 +184,16 @@ public:
 	in6_addr * GetAddr6() { return( &(m_saddr6.sin6_addr) ); }
 #endif /* HAVE_IPV6 */
 
+	void SetAFRequire( EAFRequire iWhich ) { m_iAFRequire = iWhich; }
+	EAFRequire GetAFRequire() const { return( m_iAFRequire ); }
+
 private:
 	bool			m_bIsIPv6;
 	sockaddr_in		m_saddr;
 #ifdef HAVE_IPV6
 	sockaddr_in6	m_saddr6;
 #endif /* HAVE_IPV6 */
-
+	EAFRequire		m_iAFRequire;
 };
 
 class Csock;
@@ -285,7 +297,7 @@ bool InitSSL( ECompType eCompressionType = CT_NONE );
 void SSLErrors( const char *filename, u_int iLineNum );
 #endif /* HAVE_LIBSSL */
 
-// TODO need to make this sock specific via getsockopt
+//! @todo need to make this sock specific via getsockopt
 inline int GetSockError()
 {
 #ifdef _WIN32
@@ -311,7 +323,7 @@ inline void ShutdownWin32()
 }
 #endif /* _WIN32 */
 
-// wrappers for FD_SET and such to work in templates
+//! wrappers for FD_SET and such to work in templates.
 inline void TFD_ZERO( fd_set *set )
 {
 	FD_ZERO( set );
@@ -354,11 +366,11 @@ public:
 	virtual ~CCron() {}
 
 	//! This is used by the Job Manager, and not you directly
-	void run();
+	void run( time_t & iNow );
 
 	/**
-	 * @TimeSequence	how often to run in seconds
-	 * @iMaxCycles		how many times to run, 0 makes it run forever
+	 * @param TimeSequence	how often to run in seconds
+	 * @param iMaxCycles		how many times to run, 0 makes it run forever
 	 */
 	void StartMaxCycles( int TimeSequence, u_int iMaxCycles );
 
@@ -397,6 +409,10 @@ private:
 	CS_STRING	m_sName;
 };
 
+#ifdef HAVE_LIBSSL
+typedef int (*FPCertVerifyCB)( int, X509_STORE_CTX * );
+#endif /* HAVE_LIBSSL */
+
 /**
 * @class Csock
 * @brief Basic Socket Class
@@ -420,7 +436,7 @@ public:
 	*/
 	Csock( const CS_STRING & sHostname, u_short iport, int itimeout = 60 );
 
-	// override this for accept sockets
+	//! override this for accept sockets
 	virtual Csock *GetSockObj( const CS_STRING & sHostname, u_short iPort );
 
 	virtual ~Csock();
@@ -487,6 +503,7 @@ public:
 	* Create the connection
 	*
 	* @param sBindHost the ip you want to bind to locally
+	* @param bSkipSetup if true, setting up the vhost etc is skipped
 	* @return true on success
 	*/
 	virtual bool Connect( const CS_STRING & sBindHost = "", bool bSkipSetup = false );
@@ -508,6 +525,8 @@ public:
 	*
 	* @param iPort the port to listen on
 	* @param iMaxConns the maximum amount of connections to allow
+	* @param sBindHost the vhost on which to listen
+	* @param iTimeout i dont know what this does :(
 	*/
 	virtual bool Listen( u_short iPort, int iMaxConns = SOMAXCONN, const CS_STRING & sBindHost = "", u_int iTimeout = 0 );
 
@@ -618,13 +637,13 @@ public:
 	* pushes data up on the buffer, if a line is ready
 	* it calls the ReadLine event
 	*/
-	virtual void PushBuff( const char *data, int len );
+	virtual void PushBuff( const char *data, int len, bool bStartAtZero = false );
 
 	//! This gives access to the internal buffer, if your
-	//! not going to use GetLine(), then you may want to clear this out
-	//! (if its binary data and not many '\n'
+	//! not going to use ReadLine(), then you may want to clear this out
+	//! (if its binary data and not many '\\n')
 	CS_STRING & GetInternalBuffer();
-	//! sets the max buffered threshold when enablereadline() is enabled
+	//! sets the max buffered threshold when EnableReadLine() is enabled
 	void SetMaxBufferThreshold( u_int iThreshold );
 	u_int GetMaxBufferThreshold();
 
@@ -707,6 +726,8 @@ public:
 	void SetCTXObject( SSL_CTX *sslCtx );
 	void SetFullSSLAccept();
 	SSL_SESSION * GetSSLSession();
+
+	void SetCertVerifyCB( FPCertVerifyCB pFP ) { m_pCerVerifyCB = pFP; }
 #endif /* HAVE_LIBSSL */
 
 	//! Get the send buffer
@@ -728,9 +749,7 @@ public:
 #ifdef HAVE_LIBSSL
 	X509 *getX509();
 
-	//!
 	//! Returns The Peers Public Key
-	//!
 	CS_STRING GetPeerPubKey();
 	bool RequiresClientCert();
 	void SetRequiresClientCert( bool bRequiresCert );
@@ -741,10 +760,10 @@ public:
 	virtual void SetParentSockName( const CS_STRING & sParentName );
 	const CS_STRING & GetParentSockName();
 
-	/*
+	/**
 	* sets the rate at which we can send data
-	* \param iBytes the amount of bytes we can write
-	* \param iMilliseconds the amount of time we have to rate to iBytes
+	* @param iBytes the amount of bytes we can write
+	* @param iMilliseconds the amount of time we have to rate to iBytes
 	*/
 	virtual void SetRate( u_int iBytes, unsigned long long iMilliseconds );
 
@@ -814,7 +833,7 @@ public:
 	 * Don't bother using these callbacks if you are using this class directly (without Socket Manager)
 	 * as the Socket Manager calls most of these callbacks
 	 * This WARNING event is called when your buffer for readline exceeds the warning threshold
-	 * and triggers this event. Either Override it and do nothing, or @SetMaxBufferThreshold( int )
+	 * and triggers this event. Either Override it and do nothing, or SetMaxBufferThreshold()
 	 * This event will only get called if m_bEnableReadLine is enabled
 	 */
 	virtual void ReachedMaxBuffer();
@@ -848,10 +867,22 @@ public:
 	 */
 	virtual void ConnectionRefused() {}
 	/**
-	 * This gets called every iteration of Select() if the socket is ReadPaused
+	 * This gets called every iteration of TSocketManager::Select() if the socket is ReadPaused
 	 */
 	virtual void ReadPaused() {}
 
+#ifdef HAVE_LIBSSL
+	/**
+	 * Gets called immediatly after the m_ssl member is setup and initialized, useful if you need to assign anything
+	 * to this ssl session via SSL_set_ex_data
+	 */
+	virtual void SSLFinishSetup( SSL *pSSL ) {}
+#endif /* HAVE_LIBSSL */
+
+
+	//! return how long it has been (in seconds) since the last write
+	int GetTimeSinceLastWrite() { return m_iTcount; }
+
 	//! return the data imediatly ready for read
 	virtual int GetPending();
 
@@ -888,9 +919,8 @@ public:
 	};
 	
 	/**
-	 * DNSLookup nonblocking dns lookup (when -pthread is set to compile
-	 * 
-	 * @return 0 for success, EAGAIN to check back again (same arguments as before, ETIMEDOUT on failure
+	 * nonblocking dns lookup (when -pthread is set to compile)
+	 * @return 0 for success, EAGAIN to check back again (same arguments as before), ETIMEDOUT on failure
 	 */
 	int DNSLookup( EDNSLType eDNSLType );
 
@@ -905,6 +935,15 @@ public:
 		m_bindhost.SetIPv6( b );
 	}
 
+	void SetAFRequire( CSSockAddr::EAFRequire iAFRequire )
+	{
+		m_address.SetAFRequire( iAFRequire );
+		m_bindhost.SetAFRequire( iAFRequire );
+	}
+
+	//! returns true if this socket can write its data, primarily used with rate shaping, initialize iNOW to 0 and it sets it on the first call
+	bool AllowWrite( unsigned long long & iNOW ) const;
+
 private:
 	u_short		m_iport, m_iRemotePort, m_iLocalPort;
 	int			m_iReadSock, m_iWriteSock, m_itimeout, m_iConnType, m_iTcount, m_iMethod;
@@ -925,6 +964,8 @@ private:
 	SSL_CTX				*m_ssl_ctx;
 	SSL_METHOD			*m_ssl_method;
 
+	FPCertVerifyCB		m_pCerVerifyCB;
+
 	virtual void FREE_SSL();
 	virtual void FREE_CTX();
 
@@ -966,10 +1007,10 @@ public:
 		m_iPort = iPort;
 		m_iTimeout = iTimeout;
 		m_bIsSSL = false;
-		m_bIsIPv6 = false;
 #ifdef HAVE_LIBSSL
 		m_sCipher = "HIGH";
 #endif /* HAVE_LIBSSL */
+		m_iAFrequire = CSSockAddr::RAF_ANY;
 	}
 	virtual ~CSConnection() {}
 
@@ -979,7 +1020,8 @@ public:
 	u_short GetPort() const { return( m_iPort ); }
 	int GetTimeout() const { return( m_iTimeout ); }
 	bool GetIsSSL() const { return( m_bIsSSL ); }
-	bool GetIsIPv6() const { return( m_bIsIPv6 ); }
+	CSSockAddr::EAFRequire GetAFRequire() const { return( m_iAFrequire ); }
+
 #ifdef HAVE_LIBSSL
 	const CS_STRING & GetCipher() const { return( m_sCipher ); }
 	const CS_STRING & GetPemLocation() const { return( m_sPemLocation ); }
@@ -998,10 +1040,11 @@ public:
 	void SetTimeout( int i ) { m_iTimeout = i; }
 	//! set to true to enable SSL
 	void SetIsSSL( bool b ) { m_bIsSSL = b; }
-	//! set to true to enable ipv6
-	void SetIsIPv6( bool b ) { m_bIsIPv6 = b; }
-	//! set the cipher strength to use, default is HIGH
+	//! sets the AF family type required
+	void SetAFRequire( CSSockAddr::EAFRequire iAFRequire ) { m_iAFrequire = iAFRequire; }
+
 #ifdef HAVE_LIBSSL
+	//! set the cipher strength to use, default is HIGH
 	void SetCipher( const CS_STRING & s ) { m_sCipher = s; }
 	//! set the location of the pemfile
 	void SetPemLocation( const CS_STRING & s ) { m_sPemLocation = s; }
@@ -1013,7 +1056,8 @@ protected:
 	CS_STRING	m_sHostname, m_sSockName, m_sBindHost;
 	u_short		m_iPort;
 	int			m_iTimeout;
-	bool		m_bIsSSL, m_bIsIPv6;
+	bool		m_bIsSSL;
+	CSSockAddr::EAFRequire	m_iAFrequire;
 #ifdef HAVE_LIBSSL
 	CS_STRING	m_sPemLocation, m_sPemPass, m_sCipher;
 #endif /* HAVE_LIBSSL */
@@ -1046,9 +1090,9 @@ public:
 		m_iPort = iPort;
 		m_sBindHost = sBindHost;
 		m_bIsSSL = false;
-		m_bIsIPv6 = false;
 		m_iMaxConns = SOMAXCONN;
 		m_iTimeout = 0;
+		m_iAFrequire = CSSockAddr::RAF_ANY;
 #ifdef HAVE_LIBSSL
 		m_sCipher = "HIGH";
 		m_bRequiresClientCert = false;
@@ -1060,9 +1104,9 @@ public:
 	const CS_STRING & GetSockName() const { return( m_sSockName ); }
 	const CS_STRING & GetBindHost() const { return( m_sBindHost ); }
 	bool GetIsSSL() const { return( m_bIsSSL ); }
-	bool GetIsIPv6() const { return( m_bIsIPv6 ); }
 	int GetMaxConns() const { return( m_iMaxConns ); }
 	u_int GetTimeout() const { return( m_iTimeout ); }
+	CSSockAddr::EAFRequire GetAFRequire() const { return( m_iAFrequire ); }
 #ifdef HAVE_LIBSSL
 	const CS_STRING & GetCipher() const { return( m_sCipher ); }
 	const CS_STRING & GetPemLocation() const { return( m_sPemLocation ); }
@@ -1078,12 +1122,12 @@ public:
 	void SetBindHost( const CS_STRING & sBindHost ) { m_sBindHost = sBindHost; }
 	//! set to true to enable SSL
 	void SetIsSSL( bool b ) { m_bIsSSL = b; }
-	//! set to true to enable ipv6
-	void SetIsIPv6( bool b ) { m_bIsIPv6 = b; }
 	//! set max connections as called by accept()
 	void SetMaxConns( int i ) { m_iMaxConns = i; }
 	//! sets the listen timeout. The listener class will close after timeout has been reached if not 0
 	void SetTimeout( u_int i ) { m_iTimeout = i; }
+	//! sets the AF family type required
+	void SetAFRequire( CSSockAddr::EAFRequire iAFRequire ) { m_iAFrequire = iAFRequire; }
 
 #ifdef HAVE_LIBSSL
 	//! set the cipher strength to use, default is HIGH
@@ -1098,9 +1142,10 @@ public:
 private:
 	u_short		m_iPort;
 	CS_STRING	m_sSockName, m_sBindHost;
-	bool		m_bIsSSL, m_bIsIPv6;
+	bool		m_bIsSSL;
 	int			m_iMaxConns;
 	u_int		m_iTimeout;
+	CSSockAddr::EAFRequire	m_iAFrequire;
 
 #ifdef HAVE_LIBSSL
 	CS_STRING	m_sPemLocation, m_sPemPass, m_sCipher;
@@ -1161,10 +1206,8 @@ public:
 
 	void clear()
 	{
-		for( unsigned int i = 0; i < this->size(); i++ )
-			CS_Delete( (*this)[i] );
-
-		std::vector<T *>::clear();
+		while ( this->size() )
+			DelSock( 0 );
 	}
 
 	virtual void Cleanup()
@@ -1178,7 +1221,7 @@ public:
 
 	enum EMessages
 	{
-		SUCCESS			= 0,	//! Select returned more then 1 fd ready for action
+		SUCCESS			= 0,	//! Select returned at least 1 fd ready for action
 		SELECT_ERROR	= -1,	//! An Error Happened, Probably dead socket. That socket is returned if available
 		SELECT_TIMEOUT	= -2,	//! Select Timeout
 		SELECT_TRYAGAIN	= -3	//! Select calls for you to try again
@@ -1187,14 +1230,9 @@ public:
 	/**
 	* Create a connection
 	*
-	* \param sHostname the destination
-	* \param iPort the destination port
-	* \param sSockName the Socket Name ( should be unique )
-	* \param iTimeout the amount of time to try to connect
-	* \param isSSL does the connection require a SSL layer
-	* \param sBindHost the host to bind too
-	* \param bIsIPv6 set to true to connect to an ipv6 host, requires HAVE_IPV6 at compile time to work
-	* \return true on success
+	* @param cCon the connection which should be established
+	* @param pcSock the socket used for the connectiong, can be NULL
+	* @return true on success
 	*/
 	bool Connect( const CSConnection & cCon, T * pcSock = NULL )
 	{
@@ -1207,7 +1245,9 @@ public:
 			pcSock->SetPort( cCon.GetPort() );
 			pcSock->SetTimeout( cCon.GetTimeout() );
 		}
-		pcSock->SetIPv6( cCon.GetIsIPv6() );
+
+		if( cCon.GetAFRequire() != CSSockAddr::RAF_ANY )
+			pcSock->SetAFRequire( cCon.GetAFRequire() );
 
 		// make it NON-Blocking IO
 		pcSock->BlockIO( false );
@@ -1242,7 +1282,14 @@ public:
 			pcSock = new T();
 
 		pcSock->BlockIO( false );
-		pcSock->SetIPv6( cListen.GetIsIPv6() );
+		if( cListen.GetAFRequire() != CSSockAddr::RAF_ANY )
+		{
+			pcSock->SetAFRequire( cListen.GetAFRequire() );
+#ifdef HAVE_IPV6
+			if( cListen.GetAFRequire() == CSSockAddr::RAF_INET6 )
+				pcSock->SetIPv6( true );
+#endif /* HAVE_IPV6 */
+		}
 #ifdef HAVE_LIBSSL
 		pcSock->SetSSL( cListen.GetIsSSL() );
 		if( ( cListen.GetIsSSL() ) && ( !cListen.GetPemLocation().empty() ) )
@@ -1282,10 +1329,10 @@ public:
 		return( false );
 	}
 
-	/*
-	* Best place to call this class for running, all the call backs are called
+	/**
+	* Best place to call this class for running, all the call backs are called.
 	* You should through this in your main while loop (long as its not blocking)
-	* all the events are called as needed
+	* all the events are called as needed.
 	*/
 	virtual void Loop ()
 	{
@@ -1473,8 +1520,8 @@ public:
 		Cron();
 	}
 
-	/*
-	* Make this method virtual, so you can override it when a socket is added
+	/**
+	* Make this method virtual, so you can override it when a socket is added.
 	* Assuming you might want to do some extra stuff
 	*/
 	virtual void AddSock( T *pcSock, const CS_STRING & sSockName )
@@ -1600,7 +1647,7 @@ public:
 	//! Get the Select Timeout in MICROSECONDS ( 1000 == 1 millisecond )
 	u_int GetSelectTimeout() { return( m_iSelectWait ); }
 	//! Set the Select Timeout in MICROSECODS ( 1000 == 1 millisecond )
-	//! Setting this to 0 will cause no timeout to happen, select will return instantly
+	//! Setting this to 0 will cause no timeout to happen, Select() will return instantly
 	void  SetSelectTimeout( u_int iTimeout ) { m_iSelectWait = iTimeout; }
 
 	std::vector<CCron *> & GetCrons() { return( m_vcCrons ); }
@@ -1632,13 +1679,45 @@ public:
 			CS_DEBUG( "Invalid Sock Position Requested! [" << iPos << "]" );
 			return;
 		}
-		if ( (*this)[iPos]->IsConnected() )
-			(*this)[iPos]->Disconnected(); // only call disconnected event if connected event was called (IE IsConnected was set)
 
-		CS_Delete( (*this)[iPos] );
+		Csock * pSock = (*this)[iPos];
+
+		if ( pSock->IsConnected() )
+			pSock->Disconnected(); // only call disconnected event if connected event was called (IE IsConnected was set)
+
+		m_iBytesRead += pSock->GetBytesRead();
+		m_iBytesWritten += pSock->GetBytesWritten();
+
+		CS_Delete( pSock );
 		this->erase( this->begin() + iPos );
 	}
 
+	//! Get the bytes read from all sockets current and past
+	unsigned long long GetBytesRead() const
+	{
+		// Start with the total bytes read from destroyed sockets
+		unsigned long long iRet = m_iBytesRead;
+
+		// Add in the outstanding bytes read from active sockets
+		for( u_int a = 0; a < this->size(); a++ )
+			iRet += (*this)[a]->GetBytesRead();
+
+		return( iRet );
+	}
+
+	//! Get the bytes written to all sockets current and past
+	unsigned long long GetBytesWritten() const
+	{
+		// Start with the total bytes written to destroyed sockets
+		unsigned long long iRet = m_iBytesWritten;
+
+		// Add in the outstanding bytes written to active sockets
+		for( u_int a = 0; a < this->size(); a++ )
+			iRet += (*this)[a]->GetBytesWritten();
+
+		return( iRet );
+	}
+
 private:
 	/**
 	* fills a map of socks to a message for check
@@ -1673,6 +1752,7 @@ private:
 
 		bool bHasWriteable = false;
 		bool bHasAvailSocks = false;
+		unsigned long long iNOW = 0;
 
 		for( unsigned int i = 0; i < this->size(); i++ )
 		{
@@ -1726,8 +1806,11 @@ private:
 					if ( !bIsReadPaused )
 						TFD_SET( iRSock, &rfds );
 
-					TFD_SET( iWSock, &wfds );
-					bHasWriteable = true;
+					if( pcSock->AllowWrite( iNOW ) )
+					{
+						TFD_SET( iWSock, &wfds );
+						bHasWriteable = true;
+					}
 				}
 
 			} else
@@ -1913,6 +1996,7 @@ private:
 	//! these crons get ran and checked in Loop()
 	virtual void Cron()
 	{
+		time_t iNow = 0;
 		for( unsigned int a = 0; a < m_vcCrons.size(); a++ )
 		{
 			CCron *pcCron = m_vcCrons[a];
@@ -1922,7 +2006,7 @@ private:
 				CS_Delete( pcCron );
 				m_vcCrons.erase( m_vcCrons.begin() + a-- );
 			} else
-				pcCron->run();
+				pcCron->run( iNow );
 		}
 	}
 
@@ -1937,6 +2021,8 @@ private:
 	EMessages				m_errno;
 	std::vector<CCron *>	m_vcCrons;
 	unsigned long long		m_iCallTimeouts;
+	unsigned long long		m_iBytesRead;
+	unsigned long long		m_iBytesWritten;
 	u_int					m_iSelectWait;
 };
 
