@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #ifndef _FILEUTILS_H
 #define _FILEUTILS_H
 
@@ -9,8 +11,9 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include <stdio.h>
+#include <signal.h>
 
-#include "String.h"
+#include "main.h"
 #include <vector>
 #include <map>
 using std::vector;
@@ -20,6 +23,7 @@ class CFile {
 public:
 	CFile();
 	CFile(const CString& sLongName);
+	CFile(int iFD, const CString& sLongName);
 	virtual ~CFile();
 
 	enum EOptions {
@@ -103,10 +107,11 @@ public:
 	bool Chmod(mode_t mode);
 	static bool Chmod(const CString& sFile, mode_t mode);
 	bool Seek(unsigned long uPos);
+	bool Truncate();
 	bool Open(const CString& sFileName, int iFlags, mode_t iMode = 0644);
 	bool Open(int iFlags, mode_t iMode = 0644);
 	int Read(char *pszBuffer, int iBytes);
-	bool ReadLine(CString & sData);
+	bool ReadLine(CString & sData, const CString & sDelimiter = "\n");
 	int Write(const char *pszBuffer, u_int iBytes);
 	int Write(const CString & sData);
 	void Close();
@@ -125,6 +130,7 @@ private:
 protected:
 	CString	m_sLongName;	//!< Absolute filename (m_sPath + "/" + m_sShortName)
 	CString	m_sShortName;	//!< Filename alone, without path
+	bool	m_bClose;
 };
 
 class CDir : public vector<CFile*> {
@@ -350,4 +356,38 @@ protected:
 	bool				m_bDesc;
 };
 
+//! @author imaginos@imaginos.net
+class CExecSock : public Csock {
+public:
+	CExecSock() : Csock() {
+		m_iPid = -1;
+	}
+
+	CExecSock(const CString& sExec) : Csock() {
+		Execute( sExec );
+	}
+
+	int Execute( const CString & sExec ) {
+		int iReadFD, iWriteFD;
+		m_iPid = popen2(iReadFD, iWriteFD, sExec);
+		ConnectFD(iReadFD, iWriteFD, "0.0.0.0:0");
+		return( m_iPid );
+	}
+	void Kill( int iSignal )
+	{
+		kill( m_iPid, iSignal );
+		Close();
+	}
+	virtual ~CExecSock() {
+		close2(m_iPid, GetRSock(), GetWSock());
+		SetRSock( -1 );
+		SetWSock( -1 );
+	}
+
+	int popen2(int & iReadFD, int & iWriteFD, const CString & sCommand);
+	void close2(int iPid, int iReadFD, int iWriteFD);
+
+private:
+	int			m_iPid;
+};
 #endif // !_FILEUTILS_H
