@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #ifndef _HTTPSOCK_H
 #define _HTTPSOCK_H
 
@@ -45,11 +47,13 @@ public:
 
 	// Setters
 	void SetDocRoot(const CString& s);
+	void SetLoggedIn(bool b) { m_bLoggedIn = b; }
 	// !Setters
 
 	// Getters
 	bool HasParam(const CString& sName) const;
 	CString GetParam(const CString& sName) const;
+	bool IsLoggedIn() const { return m_bLoggedIn; }
 	const CString& GetDocRoot() const;
 	const CString& GetUser() const;
 	const CString& GetPass() const;
