@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #include "znc.h"
 #include "IRCSock.h"
 #include "DCCBounce.h"
@@ -10,6 +12,8 @@ CIRCSock::CIRCSock(CUser* pUser) : Csock() {
 	m_bKeepNick = true;
 	m_bAuthed = false;
 	m_bOrigNickPending = false;
+	m_bNamesx = false;
+	m_bUHNames = false;
 	EnableReadLine();
 	m_Nick.SetIdent(pUser->GetIdent());
 	m_Nick.SetHost(pUser->GetVHost());
@@ -59,16 +63,20 @@ void CIRCSock::ReadLine(const CString& sData) {
 	DEBUG_ONLY(cout << "(" << m_pUser->GetUserName() << ") IRC -> ZNC [" << sLine << "]" << endl);
 
 #ifdef _MODULES
-	CGlobalModules& GMods = CZNC::Get().GetModules();
-	GMods.SetUser(m_pUser);
-	if (GMods.OnRaw(sLine) || m_pUser->GetModules().OnRaw(sLine)) {
-		return;
-	}
-	GMods.SetUser(NULL);
+	MODULECALL(OnRaw(sLine), m_pUser, NULL, return);
 #endif
 
 	if (strncasecmp(sLine.c_str(), "PING ", 5) == 0) {
 		PutIRC("PONG " + sLine.substr(5));
+	} else if (strncasecmp(sLine.c_str(), "ERROR ", 6) == 0) {
+		//ERROR :Closing Link: nick[24.24.24.24] (Excess Flood)
+		CString sError(sLine.substr(7));
+
+		if (sError.Left(1) == ":") {
+			sError.LeftChomp();
+		}
+
+		m_pUser->PutStatus("Error from Server [" + sError + "]");
 	} else if (sLine.WildCmp(":* * *")) { //"^:(\\S+) (\\d\\d\\d) (.*?) (.*)$", vCap)) {
 		CString sCmd = sLine.Token(1);
 
@@ -84,9 +92,9 @@ void CIRCSock::ReadLine(const CString& sData) {
 					SetTimeout(900);	// Now that we are connected, let nature take its course
 					PutIRC("WHO " + sNick);
 
-					m_pUser->StartAwayNickTimer();
+					SetNick(sNick);
 
-					VOIDMODULECALL(OnIRCConnected());
+					MODULECALL(OnIRCConnected(), m_pUser, NULL, );
 
 					m_bAuthed = true;
 					m_pUser->PutStatus("Connected!");
@@ -103,8 +111,6 @@ void CIRCSock::ReadLine(const CString& sData) {
 						}
 					}
 
-					SetNick(sNick);
-
 					m_pUser->ClearRawBuffer();
 					m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
 
@@ -134,6 +140,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				case 266:	// global users
 					m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
 					break;
+				case 422:	// MOTD File is missing
 				case 375:	// begin motd
 					m_pUser->ClearMotdBuffer();
 				case 372:	// motd
@@ -159,7 +166,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					CString sBadNick = sRest.Token(0);
 					CString sConfNick = m_pUser->GetNick().Left(uMax);
 
-					if (sNick == "*" || sNick.CaseCmp(CNick::Concat(sConfNick, m_pUser->GetAwaySuffix(), GetMaxNickLen())) == 0) {
+					if (sNick == "*") {
 						CString sAltNick = m_pUser->GetAltNick().Left(uMax);
 
 						if (sBadNick.CaseCmp(sConfNick) == 0) {
@@ -294,20 +301,65 @@ void CIRCSock::ReadLine(const CString& sData) {
 					}
 				}
 					break;
+				case 329: {
+					sRest.Trim();
+					CChan* pChan = m_pUser->FindChan(sRest.Token(0));
+
+					if (pChan) {
+						unsigned long ulDate = strtoul(sLine.Token(4).c_str(), NULL, 10);
+						pChan->SetCreationDate(ulDate);
+					}
+				}
+					break;
 				case 353: {	// NAMES
 					sRest.Trim();
 					// Todo: allow for non @+= server msgs
 					CChan* pChan = m_pUser->FindChan(sRest.Token(1));
-					if (pChan) {
-						CString sNicks = sRest.Token(2, true);
-						if (sNicks.Left(1) == ":") {
-							sNicks.LeftChomp();
-						}
+					if (!pChan) // Todo: should this still be forwarded to clients?
+						return;
 
-						pChan->AddNicks(sNicks);
+					CString sNicks = sRest.Token(2, true);
+					if (sNicks.Left(1) == ":") {
+						sNicks.LeftChomp();
 					}
+
+					pChan->AddNicks(sNicks);
+
+					CString sTmp;
+					vector<CClient*>& vClients = m_pUser->GetClients();
+					for (unsigned int a = 0; a < vClients.size(); a++) {
+						if((!m_bNamesx || vClients[a]->HasNamesx()) &&
+							(!m_bUHNames || vClients[a]->HasUHNames())) {
+							m_pUser->PutUser(sLine, vClients[a]);
+						} else {
+							sTmp = sLine.Token(0) + " ";
+							sTmp += sLine.Token(1) + " ";
+							sTmp += sLine.Token(2) + " ";
+							sTmp += sLine.Token(3) + " ";
+							sTmp += sLine.Token(4) + " :";
+							while(!sNicks.empty()) {
+								CString sNick = sNicks.Token(0);
+								sNicks = sNicks.Token(1, true);
+								if(m_bNamesx && !vClients[a]->HasNamesx()
+									&& IsPermChar(sNick[0])) {
+									while(sNick.length() > 2
+										&& IsPermChar(sNick[1])) {
+										sNick = sNick[0] + sNick.substr(2);
+									}
+								}
+
+								if(m_bUHNames && !vClients[a]->HasUHNames()) {
+									sNick = sNick.Token(0, false, "!");
+								}
+
+								sTmp += sNick + " ";
+							}
+							sTmp.RightChomp();
+							m_pUser->PutUser(sTmp, vClients[a]);
+						}
+					}
+					return;
 				}
-					break;
 				case 366: {	// end of names list
 					m_pUser->PutUser(sLine);	// First send them the raw
 
@@ -315,24 +367,6 @@ void CIRCSock::ReadLine(const CString& sData) {
 					CChan* pChan = m_pUser->FindChan(sRest.Token(0));
 
 					if (pChan) {
-						if (m_pUser->IsUserAttached()) {
-							const vector<CString>& vsBuffer = pChan->GetBuffer();
-
-							if (vsBuffer.size()) {
-								m_pUser->PutUser(":***!znc@znc.com PRIVMSG " + pChan->GetName() + " :Buffer Playback...");
-
-								for (unsigned int a = 0; a < vsBuffer.size(); a++) {
-									m_pUser->PutUser(vsBuffer[a]);
-								}
-
-								if (!pChan->KeepBuffer()) {
-									pChan->ClearBuffer();
-								}
-
-								m_pUser->PutUser(":***!znc@znc.com PRIVMSG " + pChan->GetName() + " :Playback Complete.");
-							}
-						}
-
 						if (!pChan->IsOn()) {
 							pChan->SetIsOn(true);
 							PutIRC("MODE " + pChan->GetName());
@@ -390,7 +424,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					KeepNick(true);
 				}
 
-				VOIDMODULECALL(OnNick(Nick, sNewNick, vFoundChans));
+				MODULECALL(OnNick(Nick, sNewNick, vFoundChans), m_pUser, NULL, );
 
 				if (!bIsVisible) {
 					return;
@@ -405,6 +439,10 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 				// :nick!ident@host.com QUIT :message
 
+				if (Nick.GetNick().CaseCmp(GetNick()) == 0) {
+					m_pUser->PutStatus("You quit [" + sMessage + "]");
+				}
+
 				vector<CChan*> vFoundChans;
 				const vector<CChan*>& vChans = m_pUser->GetChans();
 
@@ -424,7 +462,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					KeepNick(true);
 				}
 
-				VOIDMODULECALL(OnQuit(Nick, sMessage, vFoundChans));
+				MODULECALL(OnQuit(Nick, sMessage, vFoundChans), m_pUser, NULL, );
 
 				if (!bIsVisible) {
 					return;
@@ -443,7 +481,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				CChan* pChan = m_pUser->FindChan(sChan);
 				if (pChan) {
 					pChan->AddNick(Nick.GetNickMask());
-					VOIDMODULECALL(OnJoin(Nick.GetNickMask(), *pChan));
+					MODULECALL(OnJoin(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
 
 					if (pChan->IsDetached()) {
 						return;
@@ -458,7 +496,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				CChan* pChan = m_pUser->FindChan(sChan);
 				if (pChan) {
 					pChan->RemNick(Nick.GetNick());
-					VOIDMODULECALL(OnPart(Nick.GetNickMask(), *pChan));
+					MODULECALL(OnPart(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
 				}
 
 				// Todo: use nick compare function
@@ -492,7 +530,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 				if (pChan) {
 					pChan->RemNick(sKickedNick);
-					VOIDMODULECALL(OnKick(Nick.GetNickMask(), sKickedNick, *pChan, sMsg));
+					MODULECALL(OnKick(Nick.GetNickMask(), sKickedNick, *pChan, sMsg), m_pUser, NULL, );
 				}
 
 				if (GetNick().CaseCmp(sKickedNick) == 0) {
@@ -605,24 +643,23 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 void CIRCSock::KeepNick(bool bForce) {
 	const CString& sConfNick = m_pUser->GetNick();
-	CString sAwayNick = CNick::Concat(sConfNick, m_pUser->GetAwaySuffix(), GetMaxNickLen());
 
-	if (m_bAuthed && m_bKeepNick && (!IsOrigNickPending() || bForce) && m_pUser->GetKeepNick() && GetNick().CaseCmp(sConfNick) != 0 && GetNick().CaseCmp(sAwayNick) != 0) {
+	if (m_bAuthed && m_bKeepNick && (!IsOrigNickPending() || bForce) && m_pUser->GetKeepNick() && GetNick().CaseCmp(sConfNick) != 0) {
 		PutIRC("NICK " + sConfNick);
 		SetOrigNickPending(true);
 	}
 }
 
 bool CIRCSock::OnCTCPReply(CNick& Nick, CString& sMessage) {
-	MODULECALL(OnCTCPReply(Nick, sMessage));
+	MODULECALL(OnCTCPReply(Nick, sMessage), m_pUser, NULL, return true);
 
 	return false;
 }
 
 bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
-	MODULECALL(OnPrivCTCP(Nick, sMessage));
+	MODULECALL(OnPrivCTCP(Nick, sMessage), m_pUser, NULL, return true);
 
-	if (strncasecmp(sMessage.c_str(), "DCC ", 4) == 0 && m_pUser && m_pUser->BounceDCCs()) {
+	if (strncasecmp(sMessage.c_str(), "DCC ", 4) == 0 && m_pUser && m_pUser->BounceDCCs() && m_pUser->IsUserAttached()) {
 		// DCC CHAT chat 2453612361 44592
 		CString sType = sMessage.Token(1);
 		CString sFile = sMessage.Token(2);
@@ -631,26 +668,23 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 		unsigned long uFileSize = strtoul(sMessage.Token(5).c_str(), NULL, 10);
 
 		if (sType.CaseCmp("CHAT") == 0) {
-			if (m_pUser->IsUserAttached()) {
-				CNick FromNick(Nick.GetNickMask());
-				unsigned short uBNCPort = CDCCBounce::DCCRequest(FromNick.GetNick(), uLongIP, uPort, "", true, m_pUser, GetLocalIP(), CUtils::GetIP(uLongIP));
-
-				if (uBNCPort) {
-					m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC CHAT chat " + CString::ToString(CUtils::GetLongIP(GetLocalIP())) + " " + CString::ToString(uBNCPort) + "\001");
-				}
+			CNick FromNick(Nick.GetNickMask());
+			unsigned short uBNCPort = CDCCBounce::DCCRequest(FromNick.GetNick(), uLongIP, uPort, "", true, m_pUser, GetLocalIP(), CUtils::GetIP(uLongIP));
+			if (uBNCPort) {
+				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC CHAT chat " + CString(CUtils::GetLongIP(GetLocalIP())) + " " + CString(uBNCPort) + "\001");
 			}
 		} else if (sType.CaseCmp("SEND") == 0) {
 			// DCC SEND readme.txt 403120438 5550 1104
 			unsigned short uBNCPort = CDCCBounce::DCCRequest(Nick.GetNick(), uLongIP, uPort, sFile, false, m_pUser, GetLocalIP(), CUtils::GetIP(uLongIP));
 			if (uBNCPort) {
-				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC SEND " + sFile + " " + CString::ToString(CUtils::GetLongIP(GetLocalIP())) + " " + CString::ToString(uBNCPort) + " " + CString::ToString(uFileSize) + "\001");
+				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC SEND " + sFile + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " " + CString(uBNCPort) + " " + CString(uFileSize) + "\001");
 			}
 		} else if (sType.CaseCmp("RESUME") == 0) {
 			// Need to lookup the connection by port, filter the port, and forward to the user
 			CDCCBounce* pSock = (CDCCBounce*) CZNC::Get().GetManager().FindSockByLocalPort(atoi(sMessage.Token(3).c_str()));
 
 			if ((pSock) && (strncasecmp(pSock->GetSockName().c_str(), "DCC::", 5) == 0)) {
-				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC " + sType + " " + sFile + " " + CString::ToString(pSock->GetUserPort()) + " " + sMessage.Token(4) + "\001");
+				m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC " + sType + " " + sFile + " " + CString(pSock->GetUserPort()) + " " + sMessage.Token(4) + "\001");
 			}
 		} else if (sType.CaseCmp("ACCEPT") == 0) {
 			// Need to lookup the connection by port, filter the port, and forward to the user
@@ -661,7 +695,7 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 
 				if ((pSock) && (strncasecmp(pSock->GetSockName().c_str(), "DCC::", 5) == 0)) {
 					if (pSock->GetUserPort() == atoi(sMessage.Token(3).c_str())) {
-						m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC " + sType + " " + sFile + " " + CString::ToString(pSock->GetLocalPort()) + " " + sMessage.Token(4) + "\001");
+						m_pUser->PutUser(":" + Nick.GetNickMask() + " PRIVMSG " + GetNick() + " :\001DCC " + sType + " " + sFile + " " + CString(pSock->GetLocalPort()) + " " + sMessage.Token(4) + "\001");
 					}
 				}
 			}
@@ -674,12 +708,20 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 		CString sQuery = sMessage.Token(0).AsUpper();
 		CString sReply;
 
+		if (it == mssCTCPReplies.end()) {
+			it = mssCTCPReplies.find(sQuery);
+		}
+
 		if (it != mssCTCPReplies.end()) {
 			sReply = it->second;
 		}
 
-		if (sReply.empty() && sQuery == "VERSION" && !m_pUser->IsUserAttached()) {
-			sReply = "ZNC by prozac - http://znc.sourceforge.net";
+		if (sReply.empty() && !m_pUser->IsUserAttached()) {
+			if (sQuery == "VERSION") {
+				sReply = "ZNC by prozac - http://znc.sourceforge.net";
+			} else if (sQuery == "PING") {
+				sReply = sMessage.Token(1, true);
+			}
 		}
 
 		if (!sReply.empty()) {
@@ -692,7 +734,7 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 }
 
 bool CIRCSock::OnPrivNotice(CNick& Nick, CString& sMessage) {
-	MODULECALL(OnPrivNotice(Nick, sMessage));
+	MODULECALL(OnPrivNotice(Nick, sMessage), m_pUser, NULL, return true);
 
 	if (!m_pUser->IsUserAttached()) {
 		// If the user is detached, add to the buffer
@@ -703,7 +745,7 @@ bool CIRCSock::OnPrivNotice(CNick& Nick, CString& sMessage) {
 }
 
 bool CIRCSock::OnPrivMsg(CNick& Nick, CString& sMessage) {
-	MODULECALL(OnPrivMsg(Nick, sMessage));
+	MODULECALL(OnPrivMsg(Nick, sMessage), m_pUser, NULL, return true);
 
 	if (!m_pUser->IsUserAttached()) {
 		// If the user is detached, add to the buffer
@@ -720,7 +762,7 @@ bool CIRCSock::OnChanCTCP(CNick& Nick, const CString& sChan, CString& sMessage)
 		if (sMessage.Token(0).CaseCmp("ACTION") == 0 && (pChan->KeepBuffer() || !m_pUser->IsUserAttached())) {
 			pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :\001" + sMessage + "\001");
 		} else {
-			MODULECALL(OnChanCTCP(Nick, *pChan, sMessage));
+			MODULECALL(OnChanCTCP(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 		}
 	}
 
@@ -730,7 +772,7 @@ bool CIRCSock::OnChanCTCP(CNick& Nick, const CString& sChan, CString& sMessage)
 bool CIRCSock::OnChanNotice(CNick& Nick, const CString& sChan, CString& sMessage) {
 	CChan* pChan = m_pUser->FindChan(sChan);
 	if (pChan) {
-		MODULECALL(OnChanNotice(Nick, *pChan, sMessage));
+		MODULECALL(OnChanNotice(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 
 		if ((pChan->KeepBuffer()) || (!m_pUser->IsUserAttached())) {
 			pChan->AddBuffer(":" + Nick.GetNickMask() + " NOTICE " + sChan + " :" + sMessage);
@@ -743,7 +785,7 @@ bool CIRCSock::OnChanNotice(CNick& Nick, const CString& sChan, CString& sMessage
 bool CIRCSock::OnChanMsg(CNick& Nick, const CString& sChan, CString& sMessage) {
 	CChan* pChan = m_pUser->FindChan(sChan);
 	if (pChan) {
-		MODULECALL(OnChanMsg(Nick, *pChan, sMessage));
+		MODULECALL(OnChanMsg(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 
 		if (pChan->KeepBuffer() || !m_pUser->IsUserAttached()) {
 			pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :" + sMessage);
@@ -776,25 +818,47 @@ void CIRCSock::Connected() {
 }
 
 void CIRCSock::Disconnected() {
-	VOIDMODULECALL(OnIRCDisconnected());
+	MODULECALL(OnIRCDisconnected(), m_pUser, NULL, );
 
 	DEBUG_ONLY(cout << GetSockName() << " == Disconnected()" << endl);
-	m_pUser->PutStatus("Disconnected from IRC.  Reconnecting...");
+	if (!m_pUser->IsBeingDeleted()) {
+		m_pUser->PutStatus("Disconnected from IRC.  Reconnecting...");
+	}
+	m_pUser->ClearRawBuffer();
+	m_pUser->ClearMotdBuffer();
+
+	ResetChans();
 }
 
 void CIRCSock::SockError(int iErrno) {
 	DEBUG_ONLY(cout << GetSockName() << " == SockError(" << iErrno << ")" << endl);
-	m_pUser->PutStatus("Disconnected from IRC.  Reconnecting...");
+	if (!m_pUser->IsBeingDeleted()) {
+		m_pUser->PutStatus("Disconnected from IRC.  Reconnecting...");
+	}
+	m_pUser->ClearRawBuffer();
+	m_pUser->ClearMotdBuffer();
+
+	ResetChans();
 }
 
 void CIRCSock::Timeout() {
 	DEBUG_ONLY(cout << GetSockName() << " == Timeout()" << endl);
-	m_pUser->PutStatus("IRC connection timed out.  Reconnecting...");
+	if (!m_pUser->IsBeingDeleted()) {
+		m_pUser->PutStatus("IRC connection timed out.  Reconnecting...");
+	}
+	m_pUser->ClearRawBuffer();
+	m_pUser->ClearMotdBuffer();
+
+	ResetChans();
 }
 
 void CIRCSock::ConnectionRefused() {
 	DEBUG_ONLY(cout << GetSockName() << " == ConnectionRefused()" << endl);
-	m_pUser->PutStatus("Connection Refused.  Reconnecting...");
+	if (!m_pUser->IsBeingDeleted()) {
+		m_pUser->PutStatus("Connection Refused.  Reconnecting...");
+	}
+	m_pUser->ClearRawBuffer();
+	m_pUser->ClearMotdBuffer();
 }
 
 void CIRCSock::ParseISupport(const CString& sLine) {
@@ -834,6 +898,12 @@ void CIRCSock::ParseISupport(const CString& sLine) {
 					}
 				}
 			}
+		} else if (sName.CaseCmp("NAMESX") == 0) {
+			m_bNamesx = true;
+			PutIRC("PROTOCTL NAMESX");
+		} else if (sName.CaseCmp("UHNAMES") == 0) {
+			m_bUHNames = true;
+			PutIRC("PROTOCTL UHNAMES");
 		}
 
 		sArg = sLine.Token(i++);
@@ -862,3 +932,9 @@ CIRCSock::EChanModeArgs CIRCSock::GetModeType(unsigned char uMode) const {
 	return it->second;
 }
 
+void CIRCSock::ResetChans() {
+	for (map<CString, CChan*>::iterator a = m_msChans.begin(); a != m_msChans.end(); a++) {
+		a->second->Reset();
+	}
+}
+
