@@ -1,3 +1,7 @@
+//! @author prozac@rottenboy.com
+
+#ifdef _MODULES
+
 #include "main.h"
 #include "Modules.h"
 #include "znc.h"
@@ -24,6 +28,24 @@
 		}												\
 	}													\
 
+#define GLOBALMODCALL(func)								\
+	for (unsigned int a = 0; a < size(); a++) {			\
+		try {											\
+			CGlobalModule* pMod = (CGlobalModule*) (*this)[a];			\
+			if (m_pUser) {								\
+				pMod->SetUser(m_pUser);					\
+				pMod->func;								\
+				pMod->SetUser(NULL);					\
+			} else {									\
+				pMod->func;								\
+			}											\
+		} catch (CModule::EModException e) {			\
+			if (e == CModule::UNLOAD) {					\
+				UnloadModule((*this)[a]->GetModName());	\
+			}											\
+		}												\
+	}													\
+
 #define GLOBALMODHALTCHK(func)							\
 	bool bHaltCore = false;								\
 	for (unsigned int a = 0; a < size(); a++) {			\
@@ -124,12 +146,26 @@ CSocket::~CSocket() {
 }
 
 bool CSocket::Connect(const CString& sHostname, unsigned short uPort, bool bSSL, unsigned int uTimeout) {
-	CString sSockName = "MOD::C::" + m_pModule->GetModName() + "::" + m_pModule->GetUser()->GetUserName();
-	return m_pModule->GetManager()->Connect(sHostname, uPort, sSockName, uTimeout, bSSL, m_pModule->GetUser()->GetVHost(), (Csock*) this);
+	CUser* pUser = m_pModule->GetUser();
+	CString sSockName = "MOD::C::" + m_pModule->GetModName();
+	CString sVHost;
+
+	if (pUser) {
+		sSockName += "::" + pUser->GetUserName();
+		sVHost = m_pModule->GetUser()->GetVHost();
+	}
+
+	return m_pModule->GetManager()->Connect(sHostname, uPort, sSockName, uTimeout, bSSL, sVHost, (Csock*) this);
 }
 
 bool CSocket::Listen(unsigned short uPort, bool bSSL, unsigned int uTimeout) {
-	CString sSockName = "MOD::L::" + m_pModule->GetModName() + "::" + m_pModule->GetUser()->GetUserName();
+	CUser* pUser = m_pModule->GetUser();
+	CString sSockName = "MOD::L::" + m_pModule->GetModName();
+
+	if (pUser) {
+		sSockName += "::" + pUser->GetUserName();
+	}
+
 	return m_pModule->GetManager()->ListenAll(uPort, sSockName, bSSL, SOMAXCONN, (Csock*) this);
 }
 
@@ -191,7 +227,7 @@ CModule::~CModule() {
 	}
 
 	while (m_vSockets.size()) {
-		RemSocket(m_vSockets[0]->GetLabel());
+		RemSocket(m_vSockets[0]);
 	}
 
 	SaveRegistry();
@@ -316,8 +352,8 @@ void CModule::ListTimers() {
 
 		Table.AddRow();
 		Table.SetCell("Name", pTimer->GetName());
-		Table.SetCell("Secs", CString::ToString(pTimer->GetInterval()));
-		Table.SetCell("Cycles", ((uCycles) ? CString::ToString(uCycles) : "INF"));
+		Table.SetCell("Secs", CString(pTimer->GetInterval()));
+		Table.SetCell("Cycles", ((uCycles) ? CString(uCycles) : "INF"));
 		Table.SetCell("Description", pTimer->GetDescription());
 	}
 
@@ -414,10 +450,10 @@ void CModule::ListSockets() {
 			Table.SetCell("State", (pSocket->IsConnected() ? "Connected" : ""));
 		}
 
-		Table.SetCell("LocalPort", CString::ToString(pSocket->GetLocalPort()));
+		Table.SetCell("LocalPort", CString(pSocket->GetLocalPort()));
 		Table.SetCell("SSL", (pSocket->GetSSL() ? "yes" : "no"));
 		Table.SetCell("RemoteIP", pSocket->GetRemoteIP());
-		Table.SetCell("RemotePort", (pSocket->GetRemotePort()) ? CString::ToString(pSocket->GetRemotePort()) : CString(""));
+		Table.SetCell("RemotePort", (pSocket->GetRemotePort()) ? CString(pSocket->GetRemotePort()) : CString(""));
 	}
 
 	if (Table.size()) {
@@ -501,6 +537,10 @@ bool CModule::PutModNotice(const CString& sLine, const CString& sIdent, const CS
 // CGlobalModule //
 ///////////////////
 CModule::EModRet CGlobalModule::OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan) { return CONTINUE; }
+CModule::EModRet CGlobalModule::OnDeleteUser(CUser& User) { return CONTINUE; }
+CModule::EModRet CGlobalModule::OnLoginAttempt(CSmartPtr<CAuthBase> Auth) { return CONTINUE; }
+void CGlobalModule::OnFailedLogin(const CString& sUsername, const CString& sRemoteIP) {}
+
 
 CModules::CModules() {
 	m_pUser = NULL;
@@ -534,20 +574,20 @@ bool CModules::OnBoot() {
 	return true;
 }
 
-void CModules::OnIRCConnected() { MODUNLOADCHK(OnIRCConnected()); }
+bool CModules::OnIRCConnected() { MODUNLOADCHK(OnIRCConnected()); return false; }
 bool CModules::OnBroadcast(CString& sMessage) { MODHALTCHK(OnBroadcast(sMessage)); }
-void CModules::OnIRCDisconnected() { MODUNLOADCHK(OnIRCDisconnected()); }
+bool CModules::OnIRCDisconnected() { MODUNLOADCHK(OnIRCDisconnected()); return false; }
 bool CModules::OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize) { MODHALTCHK(OnDCCUserSend(RemoteNick, uLongIP, uPort, sFile, uFileSize)); }
-void CModules::OnChanPermission(const CNick& OpNick, const CNick& Nick, CChan& Channel, unsigned char uMode, bool bAdded, bool bNoChange) { MODUNLOADCHK(OnChanPermission(OpNick, Nick, Channel, uMode, bAdded, bNoChange)); }
-void CModules::OnOp(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnOp(OpNick, Nick, Channel, bNoChange)); }
-void CModules::OnDeop(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnDeop(OpNick, Nick, Channel, bNoChange)); }
-void CModules::OnVoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnVoice(OpNick, Nick, Channel, bNoChange)); }
-void CModules::OnDevoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnDevoice(OpNick, Nick, Channel, bNoChange)); }
-void CModules::OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs) { MODUNLOADCHK(OnRawMode(OpNick, Channel, sModes, sArgs)); }
+bool CModules::OnChanPermission(const CNick& OpNick, const CNick& Nick, CChan& Channel, unsigned char uMode, bool bAdded, bool bNoChange) { MODUNLOADCHK(OnChanPermission(OpNick, Nick, Channel, uMode, bAdded, bNoChange)); return false; }
+bool CModules::OnOp(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnOp(OpNick, Nick, Channel, bNoChange)); return false; }
+bool CModules::OnDeop(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnDeop(OpNick, Nick, Channel, bNoChange)); return false; }
+bool CModules::OnVoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnVoice(OpNick, Nick, Channel, bNoChange)); return false; }
+bool CModules::OnDevoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) { MODUNLOADCHK(OnDevoice(OpNick, Nick, Channel, bNoChange)); return false; }
+bool CModules::OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs) { MODUNLOADCHK(OnRawMode(OpNick, Channel, sModes, sArgs)); return false; }
 bool CModules::OnRaw(CString& sLine) { MODHALTCHK(OnRaw(sLine)); }
 
-void CModules::OnUserAttached() { MODUNLOADCHK(OnUserAttached()); }
-void CModules::OnUserDetached() { MODUNLOADCHK(OnUserDetached()); }
+bool CModules::OnUserAttached() { MODUNLOADCHK(OnUserAttached()); return false; }
+bool CModules::OnUserDetached() { MODUNLOADCHK(OnUserDetached()); return false; }
 bool CModules::OnUserRaw(CString& sLine) { MODHALTCHK(OnUserRaw(sLine)); }
 bool CModules::OnUserCTCPReply(CString& sTarget, CString& sMessage) { MODHALTCHK(OnUserCTCPReply(sTarget, sMessage)); }
 bool CModules::OnUserCTCP(CString& sTarget, CString& sMessage) { MODHALTCHK(OnUserCTCP(sTarget, sMessage)); }
@@ -556,11 +596,11 @@ bool CModules::OnUserNotice(CString& sTarget, CString& sMessage) { MODHALTCHK(On
 bool CModules::OnUserJoin(CString& sChannel, CString& sKey) { MODHALTCHK(OnUserJoin(sChannel, sKey)); }
 bool CModules::OnUserPart(CString& sChannel, CString& sMessage) { MODHALTCHK(OnUserPart(sChannel, sMessage)); }
 
-void CModules::OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans) { MODUNLOADCHK(OnQuit(Nick, sMessage, vChans)); }
-void CModules::OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans) { MODUNLOADCHK(OnNick(Nick, sNewNick, vChans)); }
-void CModules::OnKick(const CNick& Nick, const CString& sKickedNick, CChan& Channel, const CString& sMessage) { MODUNLOADCHK(OnKick(Nick, sKickedNick, Channel, sMessage)); }
-void CModules::OnJoin(const CNick& Nick, CChan& Channel) { MODUNLOADCHK(OnJoin(Nick, Channel)); }
-void CModules::OnPart(const CNick& Nick, CChan& Channel) { MODUNLOADCHK(OnPart(Nick, Channel)); }
+bool CModules::OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans) { MODUNLOADCHK(OnQuit(Nick, sMessage, vChans)); return false; }
+bool CModules::OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans) { MODUNLOADCHK(OnNick(Nick, sNewNick, vChans)); return false; }
+bool CModules::OnKick(const CNick& Nick, const CString& sKickedNick, CChan& Channel, const CString& sMessage) { MODUNLOADCHK(OnKick(Nick, sKickedNick, Channel, sMessage)); return false; }
+bool CModules::OnJoin(const CNick& Nick, CChan& Channel) { MODUNLOADCHK(OnJoin(Nick, Channel)); return false; }
+bool CModules::OnPart(const CNick& Nick, CChan& Channel) { MODUNLOADCHK(OnPart(Nick, Channel)); return false; }
 bool CModules::OnCTCPReply(CNick& Nick, CString& sMessage) { MODHALTCHK(OnCTCPReply(Nick, sMessage)); }
 bool CModules::OnPrivCTCP(CNick& Nick, CString& sMessage) { MODHALTCHK(OnPrivCTCP(Nick, sMessage)); }
 bool CModules::OnChanCTCP(CNick& Nick, CChan& Channel, CString& sMessage) { MODHALTCHK(OnChanCTCP(Nick, Channel, sMessage)); }
@@ -569,9 +609,9 @@ bool CModules::OnChanMsg(CNick& Nick, CChan& Channel, CString& sMessage) { MODHA
 bool CModules::OnPrivNotice(CNick& Nick, CString& sMessage) { MODHALTCHK(OnPrivNotice(Nick, sMessage)); }
 bool CModules::OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage) { MODHALTCHK(OnChanNotice(Nick, Channel, sMessage)); }
 bool CModules::OnStatusCommand(const CString& sCommand) { MODHALTCHK(OnStatusCommand(sCommand)); }
-void CModules::OnModCommand(const CString& sCommand) { MODUNLOADCHK(OnModCommand(sCommand)); }
-void CModules::OnModNotice(const CString& sMessage) { MODUNLOADCHK(OnModNotice(sMessage)); }
-void CModules::OnModCTCP(const CString& sMessage) { MODUNLOADCHK(OnModCTCP(sMessage)); }
+bool CModules::OnModCommand(const CString& sCommand) { MODUNLOADCHK(OnModCommand(sCommand)); return false; }
+bool CModules::OnModNotice(const CString& sMessage) { MODUNLOADCHK(OnModNotice(sMessage)); return false; }
+bool CModules::OnModCTCP(const CString& sMessage) { MODUNLOADCHK(OnModCTCP(sMessage)); return false; }
 
 ////////////////////
 // CGlobalModules //
@@ -580,6 +620,18 @@ bool CGlobalModules::OnConfigLine(const CString& sName, const CString& sValue, C
 	GLOBALMODHALTCHK(OnConfigLine(sName, sValue, pUser, pChan));
 }
 
+bool CGlobalModules::OnDeleteUser(CUser& User) {
+	GLOBALMODHALTCHK(OnDeleteUser(User));
+}
+
+bool CGlobalModules::OnLoginAttempt(CSmartPtr<CAuthBase> Auth) {
+	GLOBALMODHALTCHK(OnLoginAttempt(Auth));
+}
+
+void CGlobalModules::OnFailedLogin(const CString& sUsername, const CString& sRemoteIP) {
+	GLOBALMODCALL(OnFailedLogin(sUsername, sRemoteIP));
+}
+
 CModule* CModules::FindModule(const CString& sModule) const {
 	for (unsigned int a = 0; a < size(); a++) {
 		if (sModule.CaseCmp((*this)[a]->GetModName()) == 0) {
@@ -627,10 +679,7 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 	}
 
 	unsigned int uDLFlags = RTLD_LAZY;
-
-	if (!pUser) {
-		uDLFlags |= RTLD_GLOBAL;
-	}
+	uDLFlags |= (pUser) ? RTLD_LOCAL : RTLD_GLOBAL;
 
 	void* p = dlopen((sModPath).c_str(), uDLFlags);
 
@@ -916,3 +965,5 @@ void CModules::GetAvailableMods(set<CModInfo>& ssMods, bool bGlobal) {
 		}
 	}
 }
+
+#endif // _MODULES
