@@ -1,8 +1,9 @@
+//! @author prozac@rottenboy.com
+
 #include <string.h>
 #include "String.h"
 #include "FileUtils.h"
 #include "MD5.h"
-#include "main.h"
 
 const char* g_szHTMLescapes[256] = {
 	"&#0;", 0, 0, 0, 0, 0, 0, 0, 0, 0,               // 0-9
@@ -119,6 +120,19 @@ const char* g_szHTMLescapes[256] = {
 	"&yuml;",   // 255
 };
 
+CString::CString(char c) : string() { stringstream s; s << c; *this = s.str(); }
+CString::CString(unsigned char c) : string() { stringstream s; s << c; *this = s.str(); }
+CString::CString(short i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(unsigned short i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(int i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(unsigned int i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(long i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(unsigned long i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(long long i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(unsigned long long i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(double i) : string() { stringstream s; s << i; *this = s.str(); }
+CString::CString(float i) : string() { stringstream s; s << i; *this = s.str(); }
+
 inline unsigned char* CString::strnchr(const unsigned char* src, unsigned char c, unsigned int iMaxBytes, unsigned char* pFill, unsigned int* piCount) const {
     for (unsigned int a = 0; a < iMaxBytes && *src; a++, src++) {
 		if (pFill) {
@@ -260,17 +274,17 @@ CString CString::Escape_n(EEscape eFrom, EEscape eTo) const {
 	const unsigned char *p = (const unsigned char*) data();
 	unsigned int iLength = length();
 	sRet.reserve(iLength *3);
-	unsigned char ch = 0;
 	unsigned int iMaxLen = (eFrom == EHTML) ? 20 : 0;
 	unsigned char pTmp[iMaxLen +1];
 	unsigned int iCounted = 0;
 
 	for (unsigned int a = 0; a < iLength; a++, p = pStart + a) {
+		unsigned char ch = 0;
+
 		switch (eFrom) {
 			case EHTML:
 				if ((*p == '&') && (strnchr((unsigned char*) p, ';', iMaxLen, pTmp, &iCounted))) {
-					if ((iCounted >= 3) && (pTmp[1] == '#')) {
-						// do XML and HTML &#97; &#x3c
+					if ((iCounted >= 3) && (pTmp[1] == '#')) {	// do XML and HTML &#97; &#x3c
 						int base = 10;
 
 						if ((pTmp[2] & 0xDF) == 'X') {
@@ -280,8 +294,7 @@ CString CString::Escape_n(EEscape eFrom, EEscape eTo) const {
 						char* endptr = NULL;
 						unsigned int b = strtol((const char*) (pTmp +2 + (base == 16)), &endptr, base);
 
-						if ( ( *endptr == ';' ) && ( b <= 255 ) )
-						{ // incase they do something like &#7777777777;
+						if ((*endptr == ';') && (b <= 255)) { // incase they do something like &#7777777777;
 							ch = b;
 							a += iCounted;
 							break;
@@ -289,7 +302,7 @@ CString CString::Escape_n(EEscape eFrom, EEscape eTo) const {
 					}
 
 					for (unsigned int c = 0; c < 256; c++) {
-						if (strcmp(g_szHTMLescapes[c], (const char*) &pTmp) == 0) {
+						if (g_szHTMLescapes[c] && strcmp(g_szHTMLescapes[c], (const char*) &pTmp) == 0) {
 							ch = c;
 							break;
 						}
@@ -835,19 +848,6 @@ void CString::Crypt(const CString& sPass, bool bEncrypt, const CString& sIvec) {
 }
 #endif	// HAVE_LIBSSL
 
-CString CString::ToString(char c) { stringstream s; s << c; return s.str(); }
-CString CString::ToString(unsigned char c) { stringstream s; s << c; return s.str(); }
-CString CString::ToString(short i) { stringstream s; s << i; return s.str(); }
-CString CString::ToString(unsigned short i) { stringstream s; s << i; return s.str(); }
-CString CString::ToString(int i) { stringstream s; s << i; return s.str(); }
-CString CString::ToString(unsigned int i) { stringstream s; s << i; return s.str(); }
-CString CString::ToString(long i) { stringstream s; s << i; return s.str(); }
-CString CString::ToString(unsigned long i) { stringstream s; s << i; return s.str(); }
-CString CString::ToString(long long i) { stringstream s; s << i; return s.str(); }
-CString CString::ToString(unsigned long long i) { stringstream s; s << i; return s.str(); }
-CString CString::ToString(double i) { stringstream s; s << i; return s.str(); }
-CString CString::ToString(float i) { stringstream s; s << i; return s.str(); }
-
 CString CString::ToPercent(double d) {
 	char szRet[32];
 	snprintf(szRet, 32, "%.02f%%", d);
