@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #include "main.h"
 #include "znc.h"
 #include "User.h"
@@ -11,9 +13,9 @@
 
 CUser::CUser(const CString& sUserName) {
 	m_uConnectTime = 0;
-	m_sUserName = sUserName;
-	m_sNick = sUserName;
-	m_sIdent = sUserName;
+	SetUserName(sUserName);
+	m_sNick = m_sCleanUserName;
+	m_sIdent = m_sCleanUserName;
 	m_sRealName = sUserName;
 	m_uServerIdx = 0;
 #ifdef _MODULES
@@ -34,12 +36,13 @@ CUser::CUser(const CString& sUserName) {
 	m_uBufferCount = 50;
 	m_bKeepBuffer = false;
 	m_bAutoCycle = true;
-	m_pBackNickTimer = NULL;
-	m_pAwayNickTimer = NULL;
+	m_bBeingDeleted = false;
 	m_pKeepNickTimer = new CKeepNickTimer(this);
 	m_pJoinTimer = new CJoinTimer(this);
+	m_pMiscTimer = new CMiscTimer(this);
 	CZNC::Get().GetManager().AddCron(m_pKeepNickTimer);
 	CZNC::Get().GetManager().AddCron(m_pJoinTimer);
+	CZNC::Get().GetManager().AddCron(m_pMiscTimer);
 	m_sUserPath = CZNC::Get().GetUserPath() + "/" + sUserName;
 	m_sDLPath = GetUserPath() + "/downloads";
 }
@@ -54,12 +57,14 @@ CUser::~CUser() {
 	}
 
 	DelClients();
-	DelModules();
 
-	CZNC::Get().GetManager().DelCronByAddr(m_pBackNickTimer);
-	CZNC::Get().GetManager().DelCronByAddr(m_pAwayNickTimer);
+#ifdef _MODULES
+	DelModules();
+#endif
+
 	CZNC::Get().GetManager().DelCronByAddr(m_pKeepNickTimer);
 	CZNC::Get().GetManager().DelCronByAddr(m_pJoinTimer);
+	CZNC::Get().GetManager().DelCronByAddr(m_pMiscTimer);
 }
 
 #ifdef _MODULES
@@ -83,8 +88,9 @@ void CUser::DelClients() {
 bool CUser::OnBoot() {
 #ifdef _MODULES
 	return GetModules().OnBoot();
-#endif
+#else
 	return true;
+#endif
 }
 
 void CUser::IRCConnected(CIRCSock* pIRCSock) {
@@ -101,6 +107,24 @@ void CUser::IRCDisconnected() {
 	}
 }
 
+CString CUser::ExpandString(const CString& sStr) const {
+	CString sRet;
+	return ExpandString(sStr, sRet);
+}
+
+CString& CUser::ExpandString(const CString& sStr, CString& sRet) const {
+	sRet = sStr;
+	sRet.Replace("%user%", GetUserName());
+	sRet.Replace("%defnick%", GetNick());
+	sRet.Replace("%nick%", GetCurNick());
+	sRet.Replace("%altnick%", GetAltNick());
+	sRet.Replace("%ident%", GetIdent());
+	sRet.Replace("%realname%", GetRealName());
+	sRet.Replace("%vhost%", GetVHost());
+
+	return sRet;
+}
+
 void CUser::BounceAllClients() {
 	for (unsigned int a = 0; a < m_vClients.size(); a++) {
 		m_vClients[a]->BouncedOff();
@@ -116,7 +140,6 @@ void CUser::UserConnected(CClient* pClient) {
 
 	PutStatus("Another client authenticated as your user, use the 'ListClients' command to see all clients");
 	m_vClients.push_back(pClient);
-	StartBackNickTimer();
 
 	if (m_RawBuffer.IsEmpty()) {
 		pClient->PutClient(":irc.znc.com 001 " + pClient->GetNick() + " :- Welcome to ZNC -");
@@ -154,34 +177,6 @@ void CUser::UserConnected(CClient* pClient) {
 	}
 }
 
-void CUser::StartAwayNickTimer() {
-	if (!m_pAwayNickTimer) {
-		m_pAwayNickTimer = new CAwayNickTimer(this);
-		CZNC::Get().GetManager().AddCron(m_pAwayNickTimer);
-	}
-}
-
-void CUser::StartBackNickTimer() {
-	CIRCSock* pIRCSock = GetIRCSock();
-
-	if (pIRCSock) {
-		CString sConfNick = GetNick();
-
-		if (pIRCSock->GetNick().CaseCmp(CNick::Concat(sConfNick, GetAwaySuffix(), pIRCSock->GetMaxNickLen())) == 0) {
-			m_pBackNickTimer = new CBackNickTimer(this);
-			CZNC::Get().GetManager().AddCron(m_pBackNickTimer);
-		}
-	}
-}
-
-void CUser::DelAwayNickTimer() {
-	m_pAwayNickTimer = NULL;
-}
-
-void CUser::DelBackNickTimer() {
-	m_pBackNickTimer = NULL;
-}
-
 void CUser::UserDisconnected(CClient* pClient) {
 	for (unsigned int a = 0; a < m_vClients.size(); a++) {
 		if (m_vClients[a] == pClient) {
@@ -189,10 +184,6 @@ void CUser::UserDisconnected(CClient* pClient) {
 			break;
 		}
 	}
-
-	if (!IsUserAttached()) {
-		StartAwayNickTimer();
-	}
 }
 
 bool CUser::Clone(const CUser& User, CString& sErrorRet) {
@@ -216,11 +207,10 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet) {
 		SetPass(User.GetPass(), User.IsPassHashed());
 	}
 
-	SetNick(User.GetNick());
-	SetAltNick(User.GetAltNick());
-	SetIdent(User.GetIdent());
+	SetNick(User.GetNick(false));
+	SetAltNick(User.GetAltNick(false));
+	SetIdent(User.GetIdent(false));
 	SetRealName(User.GetRealName());
-	SetAwaySuffix(User.GetAwaySuffix());
 	SetStatusPrefix(User.GetStatusPrefix());
 	SetVHost(User.GetVHost());
 	SetQuitMsg(User.GetQuitMsg());
@@ -359,14 +349,6 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet) {
 	SetAdmin(User.IsAdmin());
 	// !Flags
 
-	if (!IsUserAttached()) {
-		if (GetAwaySuffix().empty()) {
-			StartBackNickTimer();
-		} else {
-			StartAwayNickTimer();
-		}
-	}
-
 	return true;
 }
 
@@ -401,8 +383,12 @@ bool CUser::IsValidUserName(const CString& sUserName) {
 		return false;
 	}
 
+	if ((*p < 'a' || *p > 'z') && (*p < 'A' || *p > 'Z')) {
+		return false;
+	}
+
 	while (*p) {
-		if (*p != '-' && *p != '_' && !isalnum(*p)) {
+		if (*p != '@' && *p != '.' && *p != '-' && *p != '_' && !isalnum(*p)) {
 			return false;
 		}
 
@@ -430,11 +416,6 @@ bool CUser::IsValid(CString& sErrMsg, bool bSkipPass) const {
 		return false;
 	}
 
-	if (m_vServers.empty()) {
-		sErrMsg = "No servers defined";
-		return false;
-	}
-
 	return true;
 }
 
@@ -493,10 +474,9 @@ bool CUser::WriteConfig(CFile& File) {
 	PrintLine(File, "RealName", GetRealName());
 	PrintLine(File, "VHost", GetVHost());
 	PrintLine(File, "QuitMsg", GetQuitMsg());
-	PrintLine(File, "AwaySuffix", GetAwaySuffix());
 	PrintLine(File, "StatusPrefix", GetStatusPrefix());
 	PrintLine(File, "ChanModes", GetDefaultChanModes());
-	PrintLine(File, "Buffer", CString::ToString(GetBufferCount()));
+	PrintLine(File, "Buffer", CString(GetBufferCount()));
 	PrintLine(File, "KeepNick", CString((GetKeepNick()) ? "true" : "false"));
 	PrintLine(File, "KeepBuffer", CString((KeepBuffer()) ? "true" : "false"));
 	PrintLine(File, "MultiClients", CString((MultiClients()) ? "true" : "false"));
@@ -740,6 +720,11 @@ CIRCSock* CUser::GetIRCSock() {
 	return (CIRCSock*) CZNC::Get().GetManager().FindSockByName("IRC::" + m_sUserName);
 }
 
+const CIRCSock* CUser::GetIRCSock() const {
+	// Todo: same as above
+	return (CIRCSock*) CZNC::Get().GetManager().FindSockByName("IRC::" + m_sUserName);
+}
+
 CString CUser::GetLocalIP() {
 	CIRCSock* pIRCSock = GetIRCSock();
 
@@ -830,7 +815,7 @@ bool CUser::ResumeFile(const CString& sRemoteNick, unsigned short uPort, unsigne
 
 			if (pSock->GetLocalPort() == uPort) {
 				if (pSock->Seek(uFileSize)) {
-					PutModule(pSock->GetModuleName(), "DCC -> [" + pSock->GetRemoteNick() + "][" + pSock->GetFileName() + "] - Attempting to resume from file position [" + CString::ToString(uFileSize) + "]");
+					PutModule(pSock->GetModuleName(), "DCC -> [" + pSock->GetRemoteNick() + "][" + pSock->GetFileName() + "] - Attempting to resume from file position [" + CString(uFileSize) + "]");
 					return true;
 				} else {
 					return false;
@@ -856,11 +841,11 @@ bool CUser::SendFile(const CString& sRemoteNick, const CString& sFileName, const
 	unsigned short uPort = CZNC::Get().GetManager().ListenAllRand("DCC::LISTEN::" + sRemoteNick, false, SOMAXCONN, pSock, 120);
 
 	if (GetNick().CaseCmp(sRemoteNick) == 0) {
-		PutUser(":" + GetStatusPrefix() + "status!znc@znc.com PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString::ToString(CUtils::GetLongIP(GetLocalIP())) + " "
-			   	+ CString::ToString(uPort) + " " + CString::ToString(pFile->GetSize()) + "\001");
+		PutUser(":" + GetStatusPrefix() + "status!znc@znc.com PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
+			   	+ CString(uPort) + " " + CString(pFile->GetSize()) + "\001");
 	} else {
-		PutIRC("PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString::ToString(CUtils::GetLongIP(GetLocalIP())) + " "
-			    + CString::ToString(uPort) + " " + CString::ToString(pFile->GetSize()) + "\001");
+		PutIRC("PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
+			    + CString(uPort) + " " + CString(pFile->GetSize()) + "\001");
 	}
 
 	PutModule(sModuleName, "DCC -> [" + sRemoteNick + "][" + pFile->GetShortName() + "] - Attempting Send.");
@@ -889,8 +874,8 @@ bool CUser::GetFile(const CString& sRemoteNick, const CString& sRemoteIP, unsign
 	return true;
 }
 
-CString CUser::GetCurNick() {
-	CIRCSock* pIRCSock = GetIRCSock();
+CString CUser::GetCurNick() const {
+	const CIRCSock* pIRCSock = GetIRCSock();
 
 	if (pIRCSock) {
 		return pIRCSock->GetNick();
@@ -903,11 +888,18 @@ CString CUser::GetCurNick() {
 	return "";
 }
 
+CString CUser::MakeCleanUserName(const CString& sUserName) {
+	return sUserName.Token(0, false, "@").Replace_n(".", "");
+}
+
 // Setters
-void CUser::SetUserName(const CString& s) { m_sUserName = s; }
+void CUser::SetUserName(const CString& s) {
+	m_sCleanUserName = CUser::MakeCleanUserName(s);
+	m_sUserName = s;
+}
+
 void CUser::SetNick(const CString& s) { m_sNick = s; }
 void CUser::SetAltNick(const CString& s) { m_sAltNick = s; }
-void CUser::SetAwaySuffix(const CString& s) { m_sAwaySuffix = s; }
 void CUser::SetIdent(const CString& s) { m_sIdent = s; }
 void CUser::SetRealName(const CString& s) { m_sRealName = s; }
 void CUser::SetVHost(const CString& s) { m_sVHost = s; }
@@ -954,10 +946,10 @@ bool CUser::SetStatusPrefix(const CString& s) {
 
 // Getters
 const CString& CUser::GetUserName() const { return m_sUserName; }
-const CString& CUser::GetNick() const { return m_sNick.empty() ? m_sUserName : m_sNick; }
-const CString& CUser::GetAltNick() const { return m_sAltNick.empty() ? m_sUserName : m_sAltNick; }
-const CString& CUser::GetAwaySuffix() const { return m_sAwaySuffix; }
-const CString& CUser::GetIdent() const { return m_sIdent.empty() ? m_sUserName : m_sIdent; }
+const CString& CUser::GetCleanUserName() const { return m_sCleanUserName; }
+const CString& CUser::GetNick(bool bAllowDefault) const { return (bAllowDefault && m_sNick.empty()) ? GetCleanUserName() : m_sNick; }
+const CString& CUser::GetAltNick(bool bAllowDefault) const { return (bAllowDefault && m_sAltNick.empty()) ? GetCleanUserName() : m_sAltNick; }
+const CString& CUser::GetIdent(bool bAllowDefault) const { return (bAllowDefault && m_sIdent.empty()) ? GetCleanUserName() : m_sIdent; }
 const CString& CUser::GetRealName() const { return m_sRealName.empty() ? m_sUserName : m_sRealName; }
 const CString& CUser::GetVHost() const { return m_sVHost; }
 const CString& CUser::GetPass() const { return m_sPass; }
