@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #ifndef _USER_H
 #define _USER_H
 
@@ -20,10 +22,9 @@ class CChan;
 class CServer;
 class CIRCSock;
 class CClient;
-class CBackNickTimer;
-class CAwayNickTimer;
 class CKeepNickTimer;
 class CJoinTimer;
+class CMiscTimer;
 
 class CUser {
 public:
@@ -48,6 +49,7 @@ public:
 	bool IsHostAllowed(const CString& sHostMask);
 	bool IsValid(CString& sErrMsg, bool bSkipPass = false) const;
 	static bool IsValidUserName(const CString& sUserName);
+	static CString MakeCleanUserName(const CString& sUserName);
 	bool IsLastServer();
 	bool ConnectPaused();
 
@@ -88,22 +90,20 @@ public:
 	void IRCConnected(CIRCSock* pIRCSock);
 	void IRCDisconnected();
 
+	CString ExpandString(const CString& sStr) const;
+	CString& ExpandString(const CString& sStr, CString& sRet) const;
+
 	bool SendFile(const CString& sRemoteNick, const CString& sFileName, const CString& sModuleName = "");
 	bool GetFile(const CString& sRemoteNick, const CString& sRemoteIP, unsigned short uRemotePort, const CString& sFileName, unsigned long uFileSize, const CString& sModuleName = "");
 	bool ResumeFile(const CString& sRemoteNick, unsigned short uPort, unsigned long uFileSize);
-	CString GetCurNick();
+	CString GetCurNick() const;
 	bool Clone(const CUser& User, CString& sErrorRet);
-	void StartAwayNickTimer();
-	void StartBackNickTimer();
-	void DelAwayNickTimer();
-	void DelBackNickTimer();
 	void BounceAllClients();
 
 	// Setters
 	void SetUserName(const CString& s);
 	void SetNick(const CString& s);
 	void SetAltNick(const CString& s);
-	void SetAwaySuffix(const CString& s);
 	void SetIdent(const CString& s);
 	void SetRealName(const CString& s);
 	void SetVHost(const CString& s);
@@ -124,16 +124,18 @@ public:
 	void SetKeepBuffer(bool b);
 	void SetAutoCycle(bool b);
 	void SetChanPrefixes(const CString& s) { m_sChanPrefixes = (s.empty()) ? "#&" : s; }
+	void SetBeingDeleted(bool b) { m_bBeingDeleted = b; }
 	// !Setters
 
 	// Getters
 	vector<CClient*>& GetClients() { return m_vClients; }
 	CIRCSock* GetIRCSock();
+	const CIRCSock* GetIRCSock() const;
 	const CString& GetUserName() const;
-	const CString& GetNick() const;
-	const CString& GetAltNick() const;
-	const CString& GetAwaySuffix() const;
-	const CString& GetIdent() const;
+	const CString& GetCleanUserName() const;
+	const CString& GetNick(bool bAllowDefault = true) const;
+	const CString& GetAltNick(bool bAllowDefault = true) const;
+	const CString& GetIdent(bool bAllowDefault = true) const;
 	const CString& GetRealName() const;
 	const CString& GetVHost() const;
 	const CString& GetPass() const;
@@ -163,14 +165,16 @@ public:
 	unsigned int GetBufferCount() const;
 	bool KeepBuffer() const;
 	bool AutoCycle() const;
+	bool IsBeingDeleted() const { return m_bBeingDeleted; }
+	bool HasServers() const { return m_vServers.size() > 0; }
 	// !Getters
 private:
 protected:
 	time_t			m_uConnectTime;
 	CString			m_sUserName;
+	CString			m_sCleanUserName;
 	CString			m_sNick;
 	CString			m_sAltNick;
-	CString			m_sAwaySuffix;
 	CString			m_sIdent;
 	CString			m_sRealName;
 	CString			m_sVHost;
@@ -188,9 +192,9 @@ protected:
 	CString			m_sDLPath;
 	// !Paths
 
-	CBuffer						m_RawBuffer;
-	CBuffer						m_MotdBuffer;
-	CBuffer						m_QueryBuffer;
+	CBuffer				m_RawBuffer;
+	CBuffer				m_MotdBuffer;
+	CBuffer				m_QueryBuffer;
 	bool				m_bIRCConnected;
 	bool				m_bMultiClients;
 	bool				m_bBounceDCCs;
@@ -201,11 +205,11 @@ protected:
 	bool				m_bAdmin;
 	bool				m_bKeepBuffer;
 	bool				m_bAutoCycle;
+	bool				m_bBeingDeleted;
 
-	CBackNickTimer*		m_pBackNickTimer;
-	CAwayNickTimer*		m_pAwayNickTimer;
 	CKeepNickTimer*		m_pKeepNickTimer;
 	CJoinTimer*			m_pJoinTimer;
+	CMiscTimer*			m_pMiscTimer;
 
 	vector<CServer*>	m_vServers;
 	vector<CChan*>		m_vChans;
