@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #include "Utils.h"
 #include "MD5.h"
 #include <sys/stat.h>
@@ -115,7 +117,7 @@ void CUtils::GenerateCert(FILE *pOut, bool bEncPrivKey, const CString& sHost) {
 		X509_free( pCert );
 		EVP_PKEY_free( pKey );
 	}
-};
+}
 #endif /* HAVE_LIBSSL */
 
 CString CUtils::GetIP(unsigned long addr) {
@@ -312,13 +314,13 @@ bool CUtils::GetNumInput(const CString& sPrompt, unsigned int& uRet, unsigned in
 		return false;
 	}
 
-	CString sDefault = (uDefault != (unsigned int) ~0) ? CString::ToString(uDefault) : "";
+	CString sDefault = (uDefault != (unsigned int) ~0) ? CString(uDefault) : "";
 	CString sNum, sHint;
 
 	if (uMax != (unsigned int) ~0) {
-		sHint = CString::ToString(uMin) + " to " + CString::ToString(uMax);
+		sHint = CString(uMin) + " to " + CString(uMax);
 	} else if (uMin > 0) {
-		sHint = CString::ToString(uMin) + " and up";
+		sHint = CString(uMin) + " and up";
 	}
 
 	while (true) {
@@ -341,6 +343,7 @@ bool CUtils::GetNumInput(const CString& sPrompt, unsigned int& uRet, unsigned in
 
 bool CUtils::GetInput(const CString& sPrompt, CString& sRet, const CString& sDefault, const CString& sHint) {
 	CString sExtra;
+	CString sInput;
 	sExtra += (!sHint.empty()) ? (" (" + sHint + ")") : "";
 	sExtra += (!sDefault.empty()) ? (" [" + sDefault + "]") : "";
 
@@ -348,14 +351,16 @@ bool CUtils::GetInput(const CString& sPrompt, CString& sRet, const CString& sDef
 	char szBuf[1024];
 	memset(szBuf, 0, 1024);
 	fgets(szBuf, 1024, stdin);
-	sRet = szBuf;
+	sInput = szBuf;
 
-	if (sRet.Right(1) == "\n") {
-		sRet.RightChomp();
+	if (sInput.Right(1) == "\n") {
+		sInput.RightChomp();
 	}
 
-	if (sRet.empty()) {
+	if (sInput.empty()) {
 		sRet = sDefault;
+	} else {
+		sRet = sInput;
 	}
 
 	return !sRet.empty();
@@ -540,7 +545,11 @@ unsigned int CTable::GetColumnWidth(unsigned int uIdx) {
 
 	for (unsigned int a = 0; a < size(); a++) {
 		map<CString, CString>* pRow = (*this)[a];
-		uRet = uRet >? (*pRow)[m_vsHeaders[uIdx]].size();
+		unsigned int uTmp = (*pRow)[m_vsHeaders[uIdx]].size();
+
+		if (uTmp > uRet) {
+			uRet = uTmp;
+		}
 	}
 
 	return uRet;
