@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #include <unistd.h>
 #include <getopt.h>
 #include <stdlib.h>
@@ -7,28 +9,30 @@
 #include "MD5.h"
 
 static struct option g_LongOpts[] = {
-	{ "help",				0,	NULL,	0 },
-	{ "version",			0,	NULL,	0 },
-	{ "makeconf",			0,	NULL,	0 },
-	{ "makepass",			0,	NULL,	0 },
+	{ "help",			no_argument,	0,	'h' },
+	{ "version",			no_argument,	0,	'v' },
+	{ "makeconf",			no_argument,	0,	'c' },
+	{ "makepass",			no_argument,	0,	's' },
 #ifdef HAVE_LIBSSL
-	{ "makepem",			0,	NULL,	0 },
-	{ "encrypt-pem",		0,	NULL, 	0 },
+	{ "makepem",			no_argument,	0,	'p' },
+	{ "encrypt-pem",		no_argument,	0, 	'e' },
 #endif /* HAVE_LIBSSL */
-	{ NULL }
+	{ "datadir",                    required_argument,	0,   'd' },
+	{ 0, 0, 0, 0 }
 };
 
 void GenerateHelp(const char *appname) {
 	CUtils::PrintMessage("USAGE: " + CString(appname) + " [options] [config]");
 	CUtils::PrintMessage("Options are:");
-	CUtils::PrintMessage("\t--help)");
-	CUtils::PrintMessage("\t--version      Output version information and exit");
-	CUtils::PrintMessage("\t--makeconf     Interactively create a new config");
-	CUtils::PrintMessage("\t--makepass     Generates a password for use in config");
+	CUtils::PrintMessage("\t-h, --help         List available command line options (this page)");
+	CUtils::PrintMessage("\t-v, --version      Output version information and exit");
+	CUtils::PrintMessage("\t-c, --makeconf     Interactively create a new config");
+	CUtils::PrintMessage("\t-s, --makepass     Generates a password for use in config");
 #ifdef HAVE_LIBSSL
-	CUtils::PrintMessage("\t--makepem      Generates a pemfile for use with SSL");
-	CUtils::PrintMessage("\t--encrypt-pem  Encrypts the pemfile");
+	CUtils::PrintMessage("\t-p, --makepem      Generates a pemfile for use with SSL");
+	CUtils::PrintMessage("\t-e, --encrypt-pem  when used along with --makepem, encrypts the private key in the pemfile");
 #endif /* HAVE_LIBSSL */
+	CUtils::PrintMessage("\t-d, --datadir      Set a different znc repository");
 }
 
 void die(int sig) {
@@ -37,7 +41,7 @@ void die(int sig) {
 	signal(SIGPIPE, SIG_DFL);
 
 #ifdef _DEBUG
-	CUtils::PrintMessage("Exiting on SIG [" + CString::ToString(sig) + "]");
+	CUtils::PrintMessage("Exiting on SIG [" + CString(sig) + "]");
 	if ((sig == SIGABRT) || (sig == SIGSEGV)) {
 		abort();
 	}
@@ -49,10 +53,10 @@ void die(int sig) {
 
 int main(int argc, char** argv, char** envp) {
 	CString sConfig;
+	CString sDataDir = "";
 
 #ifdef HAVE_LIBSSL
-	// initialize ssl, allow client to have compression enabled if desired
-	InitSSL(CT_ZLIB);
+	InitSSL();
 #endif /* HAVE_LIBSSL */
 
 	int iArg, iOptIndex = -1;
@@ -61,43 +65,41 @@ int main(int argc, char** argv, char** envp) {
 #ifdef HAVE_LIBSSL
 	bool bMakePem = false;
 	bool bEncPem = false;
+
+	while ((iArg = getopt_long(argc, argv, "hvcsped:", g_LongOpts, &iOptIndex)) != -1) {
+#else	
+
+	while ((iArg = getopt_long(argc, argv, "hvcsd:", g_LongOpts, &iOptIndex)) != -1) {
 #endif /* HAVE_LIBSSL */
-	while ((iArg = getopt_long(argc, argv, "c|h", g_LongOpts, &iOptIndex) != -1)) {
-		switch (iArg) {
-			case 1: { // long options
-				if (iOptIndex >= 0) {
-					CString sOption = Lower(g_LongOpts[iOptIndex].name);
-					if (sOption == "version") {
-						cout << CZNC::GetTag() << endl;
-						return 0;
-					} else if (sOption == "makeconf") {
-						bMakeConf = true;
-					} else if (sOption == "makepass") {
-						bMakePass = true;
+	    switch (iArg) {
+		case 'h':
+			    GenerateHelp(argv[0]);
+			    return 0;
+		case 'v':
+			    cout << CZNC::GetTag() << endl;
+			    return 0;
+		case 'c':
+			    bMakeConf = true;
+			    break;
+		case 's':
+			    bMakePass = true;
+			    break;
 #ifdef HAVE_LIBSSL
-					} else if (sOption == "makepem") {
-						bMakePem = true;
-					} else if (sOption == "encrypt-pem") {
-						bEncPem = true;
+		case 'p':
+			    bMakePem = true;
+			    break;
+		case 'e':
+			    bEncPem = true;
+			    break;
 #endif /* HAVE_LIBSSL */
-					} else if (sOption == "help") {
-						GenerateHelp(argv[0]);
-						return 0;
-					}
-				} else {
-					GenerateHelp(argv[0]);
-					return 1;
-				}
-				break;
-			}
-			case 'h':
-				GenerateHelp(argv[0]);
-				return 0;
-			default: {
-				GenerateHelp(argv[0]);
-				return 1;
-			}
-		}
+		case 'd':
+			    sDataDir = CString(optarg);
+			    break;
+		case '?':
+		default:
+			    GenerateHelp(argv[0]);
+			    return 1;
+	    }
 	}
 
 	if (optind < argc) {
@@ -108,23 +110,36 @@ int main(int argc, char** argv, char** envp) {
 
 	if (bMakeConf) {
 		CZNC& ZNC = CZNC::Get();
-		ZNC.InitDirs("");
+		ZNC.InitDirs("", sDataDir);
 		if (ZNC.WriteNewConfig(sConfig)) {
-			char* args[3];
+			char* args[5];
 
 			if (argc > 2) {
 				args[0] = argv[0];
-				args[1] = argv[optind];
-				args[2] = NULL;
+				if (!sDataDir.empty()) {
+					args[1] = "--datadir";
+					args[2] = strdup(sDataDir.c_str());
+					args[3] = argv[optind];
+					args[4] = NULL;
+				} else {
+					args[1] = argv[optind];
+					args[2] = NULL;
+				}
 			} else if (argc > 1) {
 				args[0] = argv[0];
-				args[1] = NULL;
-				args[2] = NULL;
+				if (!sDataDir.empty()) {
+					args[1] = "--datadir";
+					args[2] = strdup(sDataDir.c_str());
+					args[3] = NULL;
+				} else {
+					args[1] = NULL;
+				}
 			} else {
 				CUtils::PrintError("Unable to launch znc [Try manually restarting]");
 				return 1;
 			}
 
+			chdir(ZNC.GetCurPath().c_str());
 			if (execve(*argv, args, envp) == -1) {
 				CUtils::PrintError("Unable to launch znc [" + CString(strerror(errno)) + "]");
 				return 1;
@@ -137,12 +152,17 @@ int main(int argc, char** argv, char** envp) {
 #ifdef HAVE_LIBSSL
 	if (bMakePem) {
 		CZNC* pZNC = &CZNC::Get();
-		pZNC->InitDirs("");
-		pZNC->WritePemFile();
+		pZNC->InitDirs("", sDataDir);
+		pZNC->WritePemFile( bEncPem );
 
 		delete pZNC;
 		return 0;
 	}
+	if( bEncPem && !bMakePem ) {
+		CUtils::PrintError("--encrypt-pem should be used along with --makepem.");
+		return 1;
+	}
+
 #endif /* HAVE_LIBSSL */
 	if (bMakePass) {
 		CString sHash = CUtils::GetHashPass();
@@ -153,7 +173,7 @@ int main(int argc, char** argv, char** envp) {
 	}
 
 	CZNC* pZNC = &CZNC::Get();
-	pZNC->InitDirs(((argc) ? argv[0] : ""));
+	pZNC->InitDirs(((argc) ? argv[0] : ""), sDataDir);
 
 	if (!pZNC->ParseConfig(sConfig)) {
 		CUtils::PrintError("Unrecoverable config error.");
@@ -187,7 +207,7 @@ int main(int argc, char** argv, char** envp) {
 	}
 
 	if (iPid > 0) {
-		CUtils::PrintStatus(true, "[pid: " + CString::ToString(iPid) + "]");
+		CUtils::PrintStatus(true, "[pid: " + CString(iPid) + "]");
 
 		pZNC->WritePidFile(iPid);
 		CUtils::PrintMessage(CZNC::GetTag(false));
