@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #include "main.h"
 #include "User.h"
 #include "Nick.h"
@@ -275,7 +277,7 @@ public:
 				m_Buffer.SetLineCount(atoi(sCount.c_str()));
 			}
 
-			PutModule("Buffer count is set to [" + CString::ToString(m_Buffer.GetLineCount()) + "]");
+			PutModule("Buffer count is set to [" + CString(m_Buffer.GetLineCount()) + "]");
 		} else if (strcasecmp(sCmdName.c_str(), "DEL") == 0) {
 			Remove(atoi(sCommand.Token(1).c_str()));
 		} else {
@@ -318,7 +320,7 @@ private:
 		for (unsigned int a = 0; a < uIdx; a++, it++);
 
 		(*it).SetDisabled(bDisabled);
-		PutModule("Id " + CString::ToString(uIdx +1) + ((bDisabled) ? " Disabled" : " Enabled"));
+		PutModule("Id " + CString(uIdx +1) + ((bDisabled) ? " Disabled" : " Enabled"));
 	}
 
 	void List() {
@@ -336,7 +338,7 @@ private:
 			CWatchEntry& WatchEntry = *it;
 
 			Table.AddRow();
-			Table.SetCell("Id", CString::ToString(uIdx));
+			Table.SetCell("Id", CString(uIdx));
 			Table.SetCell("HostMask", WatchEntry.GetHostMask());
 			Table.SetCell("Target", WatchEntry.GetTarget());
 			Table.SetCell("Pattern", WatchEntry.GetPattern());
@@ -373,11 +375,11 @@ private:
 			PutModule("/msg " + GetModNick() + " ADD " + WatchEntry.GetHostMask() + " " + WatchEntry.GetTarget() + " " + WatchEntry.GetPattern());
 
 			if (WatchEntry.GetSourcesStr().size()) {
-				PutModule("/msg " + GetModNick() + " SETSOURCES " + CString::ToString(uIdx) + " " + WatchEntry.GetSourcesStr());
+				PutModule("/msg " + GetModNick() + " SETSOURCES " + CString(uIdx) + " " + WatchEntry.GetSourcesStr());
 			}
 
 			if (WatchEntry.IsDisabled()) {
-				PutModule("/msg " + GetModNick() + " DISABLE " + CString::ToString(uIdx));
+				PutModule("/msg " + GetModNick() + " DISABLE " + CString(uIdx));
 			}
 		}
 
@@ -395,7 +397,7 @@ private:
 		for (unsigned int a = 0; a < uIdx; a++, it++);
 
 		(*it).SetSources(sSources);
-		PutModule("Sources set for Id " + CString::ToString(uIdx +1) + ".");
+		PutModule("Sources set for Id " + CString(uIdx +1) + ".");
 	}
 
 	void Remove(unsigned int uIdx) {
@@ -409,7 +411,7 @@ private:
 		for (unsigned int a = 0; a < uIdx; a++, it++);
 
 		m_lsWatchers.erase(it);
-		PutModule("Id " + CString::ToString(uIdx +1) + " Removed.");
+		PutModule("Id " + CString(uIdx +1) + " Removed.");
 	}
 
 	void Help() {
