@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #include "main.h"
 #include "User.h"
 #include "Nick.h"
@@ -5,10 +7,31 @@
 #include "Chan.h"
 #include "znc.h"
 #include "HTTPSock.h"
+#include "Client.h"
 #include "Server.h"
 #include "Template.h"
 
 class CWebAdminMod;
+class CWebAdminSock;
+
+
+class CWebAdminAuth : public CAuthBase {
+public:
+	CWebAdminAuth(CWebAdminSock* pWebAdminSock, const CString& sUsername, const CString& sPassword)
+		: CAuthBase(sUsername, sPassword) {
+			m_pWebAdminSock = pWebAdminSock;
+	}
+
+	virtual ~CWebAdminAuth() {}
+
+	void SetWebAdminSock(CWebAdminSock* pWebAdminSock) { m_pWebAdminSock = pWebAdminSock; }
+	void AcceptLogin(CUser& User);
+	void RefuseLogin(const CString& sReason);
+private:
+protected:
+	CWebAdminSock*	m_pWebAdminSock;
+};
+
 
 class CWebAdminSock : public CHTTPSock {
 public:
@@ -55,15 +78,22 @@ public:
 		return "";
 	}
 
+	// Setters
+	void SetSessionUser(CUser* p) { m_pSessionUser = p; m_bAdmin = p->IsAdmin(); }
+	void SetAdmin(bool b) { m_bAdmin = b; }
+	// !Setters
+
 	virtual Csock* GetSockObj(const CString& sHost, unsigned short uPort);
 	bool IsAdmin(bool bAllowUserAdmin = true) const { return m_bAdmin; }
 
 private:
 protected:
-	CWebAdminMod*	m_pModule;
-	CUser*			m_pUser;
-	bool			m_bAdmin;
-	CTemplate		m_Template;
+	CWebAdminMod*			m_pModule;
+	CUser*					m_pUser;
+	CUser*					m_pSessionUser;
+	bool					m_bAdmin;
+	CTemplate				m_Template;
+	CSmartPtr<CAuthBase>	m_spAuth;
 };
 
 class CWebAdminMod : public CGlobalModule {
@@ -85,10 +115,27 @@ public:
 		return true;
 	}
 
-	virtual bool OnLoad(const CString& sArgs) {
+	virtual bool OnLoad(const CString& sArgStr) {
 		bool bSSL = false;
+		bool bIPv6 = false;
+		CString sArgs(sArgStr);
+		CString sOpt;
 		CString sPort;
 
+		if (sArgs.Left(1) == "-") {
+			sOpt = sArgs.Token(0);
+			sArgs = sArgs.Token(1, true);
+
+			if (sOpt.CaseCmp("-IPV6") == 0) {
+				bIPv6 = true;
+			} else if (sOpt.CaseCmp("-IPV4") == 0) {
+				bIPv6 = false;
+			} else {
+				CUtils::PrintMessage("Unknown option [" + sOpt + "] valid options are -ipv4 or -ipv6", true);
+				return false;
+			}
+		}
+
 		if (sArgs.find(" ") != CString::npos) {
 			m_sListenHost = sArgs.Token(0);
 			sPort = sArgs.Token(1);
@@ -116,7 +163,7 @@ public:
 		}
 #endif
 
-		return m_pManager->ListenHost(m_uPort, "WebAdmin::Listener", m_sListenHost, bSSL, SOMAXCONN, pListenSock);
+		return m_pManager->ListenHost(m_uPort, "WebAdmin::Listener", m_sListenHost, bSSL, SOMAXCONN, pListenSock, 0, bIPv6);
 	}
 
 	void AddSock(CWebAdminSock* pSock) {
@@ -210,6 +257,18 @@ void CWebAdminSock::PrintPage(CString& sPageRet, const CString& sTmplName) {
 }
 
 bool CWebAdminSock::OnLogin(const CString& sUser, const CString& sPass) {
+	m_spAuth = new CWebAdminAuth(this, sUser, sPass);
+
+	if (CZNC::Get().GetModules().OnLoginAttempt(m_spAuth)) {
+		if (IsLoggedIn()) {
+			return true;
+		}
+
+		PauseRead();
+
+		return false;
+	}
+
 	CUser* pUser = CZNC::Get().FindUser(GetUser());
 
 	if (pUser) {
@@ -222,6 +281,8 @@ bool CWebAdminSock::OnLogin(const CString& sUser, const CString& sPass) {
 				m_pUser = pUser;
 			}
 
+			m_pSessionUser = pUser;
+
 			return true;
 		}
 	}
@@ -242,9 +303,13 @@ void CWebAdminSock::ListUsersPage(CString& sPageRet) {
 		CUser& User = *it->second;
 
 		l["Username"] = User.GetUserName();
-		l["Clients"] = CString::ToString(User.GetClients().size());
+		l["Clients"] = CString(User.GetClients().size());
 		l["IRCNick"] = User.GetIRCNick().GetNick();
 
+		if (&User == m_pSessionUser) {
+			l["IsSelf"] = "true";
+		}
+
 		if (pServer) {
 			l["Server"] = pServer->GetName();
 		}
@@ -265,6 +330,7 @@ Csock* CWebAdminSock::GetSockObj(const CString& sHost, unsigned short uPort) {
 CWebAdminSock::CWebAdminSock(CWebAdminMod* pModule) : CHTTPSock() {
 	m_pModule = pModule;
 	m_pUser = NULL;
+	m_pSessionUser = NULL;
 	m_bAdmin = false;
 	m_pModule->AddSock(this);
 	SetDocRoot(GetSkinDir());
@@ -273,6 +339,7 @@ CWebAdminSock::CWebAdminSock(CWebAdminMod* pModule) : CHTTPSock() {
 CWebAdminSock::CWebAdminSock(CWebAdminMod* pModule, const CString& sHostname, unsigned short uPort, int iTimeout) : CHTTPSock(sHostname, uPort, iTimeout) {
 	m_pModule = pModule;
 	m_pUser = NULL;
+	m_pSessionUser = NULL;
 	m_bAdmin = false;
 	m_pModule->AddSock(this);
 	SetDocRoot(GetSkinDir());
@@ -280,6 +347,11 @@ CWebAdminSock::CWebAdminSock(CWebAdminMod* pModule, const CString& sHostname, un
 
 CWebAdminSock::~CWebAdminSock() {
 	m_pModule->SockDestroyed(this);
+
+	if (!m_spAuth.IsNull()) {
+		CWebAdminAuth* pAuth = (CWebAdminAuth*) &(*m_spAuth);
+		pAuth->SetWebAdminSock(NULL);
+	}
 }
 
 bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
@@ -333,7 +405,7 @@ bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
 		unsigned int uCounter = m_pModule->GetSwitchCounter(GetRemoteIP());
 
 		if (!uCurCnt) {
-			Redirect("/switchuser?cnt=" + CString::ToString(uCounter));
+			Redirect("/switchuser?cnt=" + CString(uCounter));
 			return false;
 		}
 
@@ -423,7 +495,12 @@ bool CWebAdminSock::OnPageRequest(const CString& sURI, CString& sPageRet) {
 			return false;
 		}
 
-		if (CZNC::Get().DeleteUser(GetParam("user"))) {
+		CString sUser = GetParam("user");
+		CUser* pUser = CZNC::Get().FindUser(sUser);
+
+		if (pUser && pUser == m_pSessionUser) {
+			GetErrorPage(sPageRet, "You are not allowed to delete yourself");
+		} else if (CZNC::Get().DeleteUser(sUser)) {
 			Redirect("/listusers");
 			return false;
 		} else {
@@ -462,7 +539,7 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 			CListener* pListener = vpListeners[c];
 			CTemplate& l = m_Template.AddRow("ListenLoop");
 
-			l["Port"] = CString::ToString(pListener->GetPort());
+			l["Port"] = CString(pListener->GetPort());
 			l["BindHost"] = pListener->GetBindHost();
 
 #ifdef HAVE_LIBSSL
@@ -609,7 +686,7 @@ bool CWebAdminSock::ChanPage(CString& sPageRet, CChan* pChan) {
 			m_Template["Edit"] = "true";
 			m_Template["Title"] = "Edit Channel" + CString(" [" + pChan->GetName() + "]");
 			m_Template["ChanName"] = pChan->GetName();
-			m_Template["BufferCount"] = CString::ToString(pChan->GetBufferCount());
+			m_Template["BufferCount"] = CString(pChan->GetBufferCount());
 			m_Template["DefModes"] = pChan->GetDefaultModes();
 
 			if (pChan->InConfig()) {
@@ -722,13 +799,12 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 			m_Template["Username"] = pUser->GetUserName();
 			m_Template["Nick"] = pUser->GetNick();
 			m_Template["AltNick"] = pUser->GetAltNick();
-			m_Template["AwaySuffix"] = pUser->GetAwaySuffix();
 			m_Template["StatusPrefix"] = pUser->GetStatusPrefix();
 			m_Template["Ident"] = pUser->GetIdent();
 			m_Template["RealName"] = pUser->GetRealName();
 			m_Template["QuitMsg"] = pUser->GetQuitMsg();
 			m_Template["DefaultChanModes"] = pUser->GetDefaultChanModes();
-			m_Template["BufferCount"] = CString::ToString(pUser->GetBufferCount());
+			m_Template["BufferCount"] = CString(pUser->GetBufferCount());
 
 			const set<CString>& ssAllowedHosts = pUser->GetAllowedHosts();
 			for (set<CString>::const_iterator it = ssAllowedHosts.begin(); it != ssAllowedHosts.end(); it++) {
@@ -756,18 +832,6 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 				}
 			}
 
-			const VCString& vsVHosts = CZNC::Get().GetVHosts();
-			for (unsigned int b = 0; b < vsVHosts.size(); b++) {
-				const CString& sVHost = vsVHosts[b];
-				CTemplate& l = m_Template.AddRow("VHostLoop");
-
-				l["VHost"] = sVHost;
-
-				if (pUser && pUser->GetVHost() == sVHost) {
-					l["Checked"] = "true";
-				}
-			}
-
 			const vector<CChan*>& Channels = pUser->GetChans();
 			for (unsigned int c = 0; c < Channels.size(); c++) {
 				CChan* pChan = Channels[c];
@@ -778,7 +842,7 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 				l["Perms"] = pChan->GetPermStr();
 				l["CurModes"] = pChan->GetModeString();
 				l["DefModes"] = pChan->GetDefaultModes();
-				l["BufferCount"] = CString::ToString(pChan->GetBufferCount());
+				l["BufferCount"] = CString(pChan->GetBufferCount());
 				l["Options"] = pChan->GetOptions();
 
 				if (pChan->InConfig()) {
@@ -791,6 +855,18 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 			m_Template["StatusPrefix"] = "*";
 		}
 
+		const VCString& vsVHosts = CZNC::Get().GetVHosts();
+		for (unsigned int b = 0; b < vsVHosts.size(); b++) {
+			const CString& sVHost = vsVHosts[b];
+			CTemplate& l = m_Template.AddRow("VHostLoop");
+
+			l["VHost"] = sVHost;
+
+			if (pUser && pUser->GetVHost() == sVHost) {
+				l["Checked"] = "true";
+			}
+		}
+
 		set<CModInfo> ssUserMods;
 		CZNC::Get().GetModules().GetAvailableMods(ssUserMods);
 
@@ -873,13 +949,12 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 
 	if (!pUser) {
 		// Add User Submission
-		if (!pNewUser->IsValid(sErr)) {
+		if (!CZNC::Get().AddUser(pNewUser, sErr)) {
 			delete pNewUser;
 			GetErrorPage(sPageRet, "Invalid submission [" + sErr + "]");
 			return true;
 		}
 
-		CZNC::Get().AddUser(pNewUser);
 		if (!CZNC::Get().WriteConfig()) {
 			GetErrorPage(sPageRet, "User added, but config was not written");
 			return true;
@@ -999,7 +1074,6 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 
 	sArg = GetParam("nick"); if (!sArg.empty()) { pNewUser->SetNick(sArg); }
 	sArg = GetParam("altnick"); if (!sArg.empty()) { pNewUser->SetAltNick(sArg); }
-	sArg = GetParam("awaysuffix"); if (!sArg.empty()) { pNewUser->SetAwaySuffix(sArg); }
 	sArg = GetParam("statusprefix"); if (!sArg.empty()) { pNewUser->SetStatusPrefix(sArg); }
 	sArg = GetParam("ident"); if (!sArg.empty()) { pNewUser->SetIdent(sArg); }
 	sArg = GetParam("realname"); if (!sArg.empty()) { pNewUser->SetRealName(sArg); }
@@ -1036,4 +1110,18 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 	return pNewUser;
 }
 
+void CWebAdminAuth::AcceptLogin(CUser& User) {
+	if (m_pWebAdminSock) {
+		m_pWebAdminSock->SetSessionUser(&User);
+		m_pWebAdminSock->SetLoggedIn(true);
+		m_pWebAdminSock->UnPauseRead();
+	}
+}
+
+void CWebAdminAuth::RefuseLogin(const CString& sReason) {
+	if (m_pWebAdminSock) {
+		m_pWebAdminSock->UnPauseRead();
+	}
+}
+
 GLOBALMODULEDEFS(CWebAdminMod, "Dynamic configuration of users/settings through a web browser")
