@@ -1,3 +1,5 @@
+//! @author prozac@rottenboy.com
+
 #include "znc.h"
 #include "User.h"
 #include "Server.h"
@@ -17,14 +19,16 @@ CZNC::CZNC() {
 #ifdef _MODULES
 	m_pModules = new CGlobalModules();
 #endif
-	m_bISpoofLocked = false;
+	m_pISpoofLockFile = NULL;
 	SetISpoofFormat(""); // Set ISpoofFormat to default
 }
 
 CZNC::~CZNC() {
+	if(m_pISpoofLockFile)
+		ReleaseISpoof();
+
 #ifdef _MODULES
 	m_pModules->UnloadAll();
-	delete m_pModules;
 
 	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
 		a->second->GetModules().UnloadAll();
@@ -35,8 +39,16 @@ CZNC::~CZNC() {
 		delete m_vpListeners[b];
 	}
 
+	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
+		a->second->SetBeingDeleted(true);
+	}
+
 	m_Manager.Cleanup();
 	DeleteUsers();
+
+#ifdef _MODULES
+	delete m_pModules;
+#endif
 }
 
 CString CZNC::GetTag(bool bIncludeVersion) {
@@ -76,6 +88,14 @@ int CZNC::Loop() {
 		if (m_ssDelUsers.size()) {
 			for (set<CUser*>::iterator it = m_ssDelUsers.begin(); it != m_ssDelUsers.end(); it++) {
 				CUser* pUser = *it;
+				pUser->SetBeingDeleted(true);
+
+#ifdef _MODULES
+				if (GetModules().OnDeleteUser(*pUser)) {
+					pUser->SetBeingDeleted(false);
+					continue;
+				}
+#endif
 				m_msUsers.erase(pUser->GetUserName());
 
 				CIRCSock* pIRCSock = pUser->GetIRCSock();
@@ -98,7 +118,7 @@ int CZNC::Loop() {
 
 		m_Manager.Loop();
 
-		if (m_bISpoofLocked) {
+		if (m_pISpoofLockFile != NULL) {
 			continue;
 		}
 
@@ -118,7 +138,7 @@ int CZNC::Loop() {
 
 		CIRCSock* pIRCSock = (CIRCSock*) m_Manager.FindSockByName(sSockName);
 
-		if (!pIRCSock) {
+		if (!pIRCSock && pUser->HasServers()) {
 			if (pUser->ConnectPaused() && pUser->IsLastServer()) {
 				continue;
 			}
@@ -129,28 +149,14 @@ int CZNC::Loop() {
 				continue;
 			}
 
-			if (!m_sISpoofFile.empty()) {
-				CFile File(m_sISpoofFile);
-
-				if (File.Open(O_RDONLY)) {
-					char buf[1024];
-					memset((char*) buf, 0, 1024);
-					File.Read(buf, 1023);
-					File.Close();
-					m_sOrigISpoof = buf;
-				}
-
-				if (File.Open(O_WRONLY | O_TRUNC | O_CREAT)) {
-					CString sData = m_sISpoofFormat.Token(0, false, "%") + pUser->GetIdent() + m_sISpoofFormat.Token(1, true, "%");
-					File.Write(sData + "\n");
-					File.Close();
-				}
-
-				m_bISpoofLocked = true;
+			if(!WriteISpoof(pUser)) {
+				DEBUG_ONLY(cout << "ISpoof could not be written" << endl);
+				pUser->PutStatus("ISpoof could not be written, retrying...");
+				continue;
 			}
 
 			DEBUG_ONLY(cout << "User [" << pUser->GetUserName() << "] is connecting to [" << pServer->GetName() << ":" << pServer->GetPort() << "] ..." << endl);
-			pUser->PutStatus("Attempting to connect to [" + pServer->GetName() + ":" + CString::ToString(pServer->GetPort()) + "] ...");
+			pUser->PutStatus("Attempting to connect to [" + pServer->GetName() + ":" + CString(pServer->GetPort()) + "] ...");
 
 			pIRCSock = new CIRCSock(pUser);
 			pIRCSock->SetPass(pServer->GetPass());
@@ -173,19 +179,53 @@ int CZNC::Loop() {
 	return 0;
 }
 
-void CZNC::ReleaseISpoof() {
-	if (!m_sISpoofFile.empty()) {
-		CFile File(m_sISpoofFile);
+bool CZNC::WriteISpoof(CUser* pUser) {
+	if(m_pISpoofLockFile != NULL)
+		return false;
 
-		if (File.Open(O_WRONLY | O_TRUNC | O_CREAT)) {
+	if (!m_sISpoofFile.empty()) {
+		m_pISpoofLockFile = new CLockFile;
+		if(!m_pISpoofLockFile->TryExLock(m_sISpoofFile, 50, true)) {
+			delete m_pISpoofLockFile;
+			m_pISpoofLockFile = NULL;
+			return false;
+		}
+
+		CFile File(m_pISpoofLockFile->GetFD(), m_pISpoofLockFile->GetFileName());
+
+		char buf[1024];
+		memset((char*) buf, 0, 1024);
+		File.Read(buf, 1023);
+		m_sOrigISpoof = buf;
+
+		if (!File.Seek(0) || !File.Truncate()) {
+			delete m_pISpoofLockFile;
+			m_pISpoofLockFile = NULL;
+			return false;
+		}
+
+		CString sData = m_sISpoofFormat.Token(0, false, "%") + pUser->GetIdent() + m_sISpoofFormat.Token(1, true, "%");
+		File.Write(sData + "\n");
+	}
+	return true;
+}
+
+void CZNC::ReleaseISpoof() {
+	if(m_pISpoofLockFile == NULL)
+		return;
+
+	if (!m_sISpoofFile.empty()) {
+		CFile File(m_pISpoofLockFile->GetFD(), m_pISpoofLockFile->GetFileName());
+
+		if (File.Seek(0) && File.Truncate()) {
 			File.Write(m_sOrigISpoof);
-			File.Close();
 		}
 
 		m_sOrigISpoof = "";
 	}
 
-	m_bISpoofLocked = false;
+	delete m_pISpoofLockFile;
+	m_pISpoofLockFile = NULL;
 }
 
 bool CZNC::WritePidFile(int iPid) {
@@ -194,7 +234,7 @@ bool CZNC::WritePidFile(int iPid) {
 		CUtils::PrintAction("Writing pid file [" + m_sPidFile + "]");
 
 		if (File.Open(O_WRONLY | O_TRUNC | O_CREAT)) {
-			File.Write(CString::ToString(iPid) + "\n");
+			File.Write(CString(iPid) + "\n");
 			File.Close();
 			CUtils::PrintStatus(true);
 			return true;
@@ -206,7 +246,7 @@ bool CZNC::WritePidFile(int iPid) {
 	return false;
 }
 
-bool CZNC::WritePemFile() {
+bool CZNC::WritePemFile( bool bEncPem ) {
 #ifndef HAVE_LIBSSL
 	CUtils::PrintError("ZNC was not compiled with ssl support.");
 	return false; 
@@ -234,7 +274,7 @@ bool CZNC::WritePemFile() {
 		return false;
 	}
 
-	CUtils::GenerateCert(f, false, sHost);
+	CUtils::GenerateCert(f, bEncPem, sHost);
 	fclose(f);
 
 	CUtils::PrintStatus(true);
@@ -244,6 +284,7 @@ bool CZNC::WritePemFile() {
 
 void CZNC::DeleteUsers() {
 	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
+		a->second->SetBeingDeleted(true);
 		delete a->second;
 	}
 
@@ -271,7 +312,7 @@ bool CZNC::IsHostAllowed(const CString& sHostMask) {
 	return false;
 }
 
-void CZNC::InitDirs(const CString& sArgvPath) {
+void CZNC::InitDirs(const CString& sArgvPath, const CString& sDataDir) {
 	char buf[PATH_MAX];
 	getcwd(buf, PATH_MAX);
 
@@ -290,7 +331,11 @@ void CZNC::InitDirs(const CString& sArgvPath) {
 		m_sHomePath = m_sCurPath;
 	}
 
-	m_sZNCPath = m_sHomePath + "/.znc";
+	if (sDataDir.empty()) {
+	    m_sZNCPath = m_sHomePath + "/.znc";
+	} else {
+	    m_sZNCPath = sDataDir;
+	}
 
 	// Other dirs that we use
 	m_sConfPath = m_sZNCPath + "/configs";
@@ -321,7 +366,7 @@ CString CZNC::ExpandConfigPath(const CString& sConfigFile) {
 bool CZNC::WriteConfig() {
 	CFile File(m_sConfigFile);
 
-	if (!File.Copy(GetConfBackupPath() + "/" + File.GetShortName() + "-" + CString::ToString(time(NULL)))) {
+	if (!File.Copy(GetConfBackupPath() + "/" + File.GetShortName() + "-" + CString(time(NULL)))) {
 		return false;
 	}
 
@@ -339,7 +384,7 @@ bool CZNC::WriteConfig() {
 
 		CString s6 = (pListener->IsIPV6()) ? "6" : " ";
 
-		File.Write("Listen" + s6 + "      = " + sHostPortion + CString((pListener->IsSSL()) ? "+" : "") + CString::ToString(pListener->GetPort()) + "\r\n");
+		File.Write("Listen" + s6 + "      = " + sHostPortion + CString((pListener->IsSSL()) ? "+" : "") + CString(pListener->GetPort()) + "\r\n");
 	}
 
 	if (!m_sISpoofFile.empty()) {
@@ -440,7 +485,7 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 		sListenHost += " ";
 	}
 
-	vsLines.push_back("Listen" + s6 + "    = " + sListenHost + sSSL + CString::ToString(uPort));
+	vsLines.push_back("Listen" + s6 + "    = " + sListenHost + sSSL + CString(uPort));
 	// !Listen
 
 #ifdef _MODULES
@@ -511,7 +556,7 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 			vsLines.push_back("\tAdmin      = false");
 		}
 
-		CUtils::GetInput("Nick", sNick, sUser);			vsLines.push_back("\tNick       = " + sNick);
+		CUtils::GetInput("Nick", sNick, CUser::MakeCleanUserName(sUser));			vsLines.push_back("\tNick       = " + sNick);
 		CUtils::GetInput("Alt Nick", sAnswer, sNick + "_");	if (!sAnswer.empty()) { vsLines.push_back("\tAltNick    = " + sAnswer); }
 		CUtils::GetInput("Ident", sAnswer, sNick);			vsLines.push_back("\tIdent      = " + sAnswer);
 		CUtils::GetInput("Real Name", sAnswer, "Got ZNC?");	vsLines.push_back("\tRealName   = " + sAnswer);
@@ -526,7 +571,7 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 
 		unsigned int uBufferCount = 0;
 
-		CUtils::GetNumInput("Number of lines to buffer per channel", uBufferCount, 0, ~0, 50);	if (uBufferCount) { vsLines.push_back("\tBuffer     = " + CString::ToString(uBufferCount)); }
+		CUtils::GetNumInput("Number of lines to buffer per channel", uBufferCount, 0, ~0, 50);	if (uBufferCount) { vsLines.push_back("\tBuffer     = " + CString(uBufferCount)); }
 		if (CUtils::GetBoolInput("Would you like your buffer to be sticky?", true)) {
 			vsLines.push_back("\tKeepBuffer = true");
 		} else {
@@ -598,7 +643,7 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 			bSSL = CUtils::GetBoolInput("Does this server use SSL? (probably no)", false);
 #endif
 
-			vsLines.push_back("\tServer     = " + sHost + ((bSSL) ? " +" : " ") + CString::ToString(uPort) + " " + sPass);
+			vsLines.push_back("\tServer     = " + sHost + ((bSSL) ? " +" : " ") + CString(uPort) + " " + sPass);
 		} while (CUtils::GetBoolInput("Would you like to add another server?", false));
 
 		vsLines.push_back("");
@@ -678,7 +723,7 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 	CUtils::PrintMessage("as the irc server password like so.. user:pass.", true);
 	CUtils::PrintMessage("");
 	CUtils::PrintMessage("Try something like this in your IRC client...", true);
-	CUtils::PrintMessage("/server <znc_server_ip> " + CString::ToString(uPort) + " " + sUser + ":<pass>", true);
+	CUtils::PrintMessage("/server <znc_server_ip> " + CString(uPort) + " " + sUser + ":<pass>", true);
 	CUtils::PrintMessage("");
 
 	m_LockFile.UnLock();
@@ -719,8 +764,11 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 	bool bCommented = false;	// support for /**/ style comments
 	CUser* pUser = NULL;	// Used to keep track of which user block we are in
 	CChan* pChan = NULL;	// Used to keep track of which chan block we are in
+	unsigned int uLineNum = 0;
 
 	while (File.ReadLine(sLine)) {
+		uLineNum++;
+
 		while ((sLine.Right(1) == "\r") || (sLine.Right(1) == "\n")) {
 			sLine.Trim();
 		}
@@ -769,13 +817,13 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 					} else if (sTag.CaseCmp("User") == 0) {
 						CString sErr;
 
-						if (!pUser->IsValid(sErr)) {
+						if (!AddUser(pUser, sErr)) {
 							CUtils::PrintError("Invalid user [" + pUser->GetUserName() + "] " + sErr);
+							pUser->SetBeingDeleted(true);
+							delete pUser;
 							return false;
 						}
 
-						AddUser(pUser);
-
 						pUser = NULL;
 						continue;
 					}
@@ -880,7 +928,7 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 						pUser->SetAltNick(sValue);
 						continue;
 					} else if (sName.CaseCmp("AwaySuffix") == 0) {
-						pUser->SetAwaySuffix(sValue);
+						CUtils::PrintMessage("WARNING: AwaySuffix has been depricated, instead try -> LoadModule = awaynick %nick%_" + sValue);
 						continue;
 					} else if (sName.CaseCmp("Pass") == 0) {
 						if (sValue.Right(1) == "-") {
@@ -998,7 +1046,7 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 					}
 
 					unsigned short uPort = strtol(sPort.c_str(), NULL, 10);
-					CUtils::PrintAction("Binding to port [" + CString((bSSL) ? "+" : "") + CString::ToString(uPort) + "]" + sHostComment + sIPV6Comment);
+					CUtils::PrintAction("Binding to port [" + CString((bSSL) ? "+" : "") + CString(uPort) + "]" + sHostComment + sIPV6Comment);
 
 #ifndef HAVE_IPV6
 					if (bIPV6) {
@@ -1024,7 +1072,7 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 							return false;
 						}
 
-						CUtils::PrintAction("Binding to port [+" + CString::ToString(uPort) + "]" + sHostComment + sIPV6Comment);
+						CUtils::PrintAction("Binding to port [+" + CString(uPort) + "]" + sHostComment + sIPV6Comment);
 					}
 #endif
 					if (!uPort) {
@@ -1069,6 +1117,10 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 					m_sISpoofFormat = sValue;
 					continue;
 				} else if (sName.CaseCmp("ISpoofFile") == 0) {
+					if(sValue.Left(2) == "~/") {
+						sValue.LeftChomp(2);
+						sValue = GetHomePath() + "/" + sValue;
+					}
 					m_sISpoofFile = sValue;
 					continue;
 				} else if (sName.CaseCmp("MOTD") == 0) {
@@ -1092,7 +1144,7 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 			}
 		}
 
-		CUtils::PrintError("Unhandled line in config: [" + sLine + "]");
+		CUtils::PrintError("Unhandled line " + CString(uLineNum) + " in config: [" + sLine + "]");
 		return false;
 	}
 
@@ -1139,8 +1191,7 @@ void CZNC::Broadcast(const CString& sMessage, CUser* pUser) {
 		if (a->second != pUser) {
 			CString sMsg = sMessage;
 #ifdef _MODULES
-			CUser* m_pUser = a->second;	// This is a semi-hack because MODULECALLCONT below expects the user to be stored in m_pUser
-			MODULECALLCONT(OnBroadcast(sMsg));
+			MODULECALL(OnBroadcast(sMsg), a->second, NULL, continue);
 #endif
 			a->second->PutStatusNotice("*** " + sMsg);
 		}
@@ -1192,15 +1243,13 @@ bool CZNC::DeleteUser(const CString& sUsername) {
 	return true;
 }
 
-bool CZNC::AddUser(CUser* pUser) {
-	CString sErr;
-
-	if (pUser->IsValid(sErr)) {
+bool CZNC::AddUser(CUser* pUser, CString& sErrorRet) {
+	if (pUser->IsValid(sErrorRet)) {
 		m_msUsers[pUser->GetUserName()] = pUser;
 		return true;
 	}
 
-	DEBUG_ONLY(cout << "Invalid user [" << pUser->GetUserName() << "] - [" << sErr << "]" << endl);
+	DEBUG_ONLY(cout << "Invalid user [" << pUser->GetUserName() << "] - [" << sErrorRet << "]" << endl);
 	return false;
 }
 
